Class: Solargraph::Convention::Base
- Inherits:
-
Object
- Object
- Solargraph::Convention::Base
- Defined in:
- lib/solargraph/convention/base.rb
Overview
The base class for Conventions.
A Convention provides Environs that customize ApiMaps with additional
pins and other information. Subclasses should implement the local and
global methods as necessary.
Direct Known Subclasses
Constant Summary collapse
Instance Method Summary collapse
-
#global(doc_map) ⇒ Environ
The Environ for a DocMap.
-
#local(source_map) ⇒ Environ
The Environ for a source map.
-
#object(api_map, rooted_tag, scope, visibility, deep, skip, no_core) ⇒ Environ
Provides any additional method pins based on e the described object.
Instance Method Details
#global(doc_map) ⇒ Environ
The Environ for a DocMap. Subclasses can override this method.
28 29 30 |
# File 'lib/solargraph/convention/base.rb', line 28 def global doc_map EMPTY_ENVIRON end |
#local(source_map) ⇒ Environ
The Environ for a source map. Subclasses can override this method.
19 20 21 |
# File 'lib/solargraph/convention/base.rb', line 19 def local source_map EMPTY_ENVIRON end |
#object(api_map, rooted_tag, scope, visibility, deep, skip, no_core) ⇒ Environ
Provides any additional method pins based on e the described object.
44 45 46 47 |
# File 'lib/solargraph/convention/base.rb', line 44 def object api_map, rooted_tag, scope, visibility, deep, skip, no_core EMPTY_ENVIRON end |