Class: Solargraph::SourceMap::Data
- Inherits:
-
Object
- Object
- Solargraph::SourceMap::Data
- Defined in:
- lib/solargraph/source_map/data.rb
Instance Method Summary collapse
-
#initialize(source) ⇒ Data
constructor
A new instance of Data.
-
#locals ⇒ Array<Solargraph::Pin::LocalVariable>
@sg-ignore Translate to something flow sensitive typing understands.
-
#pins ⇒ Array<Solargraph::Pin::Base>
@sg-ignore Translate to something flow sensitive typing understands @sg-ignore github.com/castwide/solargraph/pull/1100.
Constructor Details
#initialize(source) ⇒ Data
Returns a new instance of Data.
7 8 9 |
# File 'lib/solargraph/source_map/data.rb', line 7 def initialize source @source = source end |
Instance Method Details
#locals ⇒ Array<Solargraph::Pin::LocalVariable>
@sg-ignore Translate to something flow sensitive typing understands
21 22 23 24 |
# File 'lib/solargraph/source_map/data.rb', line 21 def locals generate @locals || [] end |
#pins ⇒ Array<Solargraph::Pin::Base>
@sg-ignore Translate to something flow sensitive typing understands @sg-ignore github.com/castwide/solargraph/pull/1100
14 15 16 17 |
# File 'lib/solargraph/source_map/data.rb', line 14 def pins generate @pins || [] end |