Class: Solargraph::YardMap
- Inherits:
-
Object
- Object
- Solargraph::YardMap
- Defined in:
- lib/solargraph/yard_map.rb
Defined Under Namespace
Classes: Cache
Instance Attribute Summary collapse
-
#workspace ⇒ Object
readonly
Returns the value of attribute workspace.
Instance Method Summary collapse
- #at(signature) ⇒ Object
- #document(query) ⇒ Object
- #find_fully_qualified_namespace(namespace, scope) ⇒ Object
- #gem_names ⇒ Object
- #get_constants(namespace, scope = '') ⇒ Array<Suggestion>
- #get_instance_methods(namespace, scope = '', visibility: [:public]) ⇒ Array<Suggestion>
- #get_methods(namespace, scope = '', visibility: [:public]) ⇒ Array<Suggestion>
- #include_bundled_gems ⇒ Object
-
#initialize(required: [], workspace: nil, with_bundled: false) ⇒ YardMap
constructor
A new instance of YardMap.
- #load_yardoc(y) ⇒ Object
- #objects(path, space = '') ⇒ Object
- #resolve(signature, scope) ⇒ Object
- #search(query) ⇒ Object
- #yardocs ⇒ Array<String>
Constructor Details
#initialize(required: [], workspace: nil, with_bundled: false) ⇒ YardMap
Returns a new instance of YardMap.
13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 |
# File 'lib/solargraph/yard_map.rb', line 13 def initialize required: [], workspace: nil, with_bundled: false @workspace = workspace unless workspace.nil? wsy = File.join(workspace, '.yardoc') yardocs.push wsy if File.exist?(wsy) end used = [] required.each { |r| if workspace.nil? or !File.exist?(File.join workspace, 'lib', "#{r}.rb") g = r.split('/').first unless used.include?(g) used.push g gy = YARD::Registry.yardoc_file_for_gem(g) if gy.nil? STDERR.puts "Required path not found: #{r}" else STDERR.puts "Adding #{gy}" yardocs.unshift gy end end end } yardocs.push File.join(Dir.home, '.solargraph', 'cache', '2.0.0', 'yardoc') yardocs.uniq! include_bundled_gems if with_bundled cache_core end |
Instance Attribute Details
#workspace ⇒ Object (readonly)
Returns the value of attribute workspace.
11 12 13 |
# File 'lib/solargraph/yard_map.rb', line 11 def workspace @workspace end |
Instance Method Details
#at(signature) ⇒ Object
138 139 140 141 142 143 144 145 146 147 |
# File 'lib/solargraph/yard_map.rb', line 138 def at signature yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? obj = yard.at(signature) return obj unless obj.nil? end } nil end |
#document(query) ⇒ Object
91 92 93 94 95 96 97 98 99 100 101 |
# File 'lib/solargraph/yard_map.rb', line 91 def document query found = [] yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? obj = yard.at query found.push obj unless obj.nil? end } found end |
#find_fully_qualified_namespace(namespace, scope) ⇒ Object
253 254 255 256 257 258 259 260 261 262 |
# File 'lib/solargraph/yard_map.rb', line 253 def find_fully_qualified_namespace namespace, scope yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? obj = find_first_resolved_namespace(yard, namespace, scope) return obj.path unless obj.nil? or !obj.kind_of?(YARD::CodeObjects::NamespaceObject) end } nil end |
#gem_names ⇒ Object
249 250 251 |
# File 'lib/solargraph/yard_map.rb', line 249 def gem_names Gem::Specification.map{ |s| s.name }.uniq end |
#get_constants(namespace, scope = '') ⇒ Array<Suggestion>
104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 |
# File 'lib/solargraph/yard_map.rb', line 104 def get_constants namespace, scope = '' cached = cache.get_constants(namespace, scope) return cached unless cached.nil? consts = [] result = [] yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? ns = nil if scope == '' ns = yard.at(namespace) else ns = yard.resolve(P(scope), namespace) end consts += ns.children unless ns.nil? end } consts.each { |c| detail = nil kind = nil if c.kind_of?(YARD::CodeObjects::ClassObject) detail = 'Class' kind = Suggestion::CLASS elsif c.kind_of?(YARD::CodeObjects::ModuleObject) detail = 'Module' kind = Suggestion::MODULE end result.push Suggestion.new(c.to_s.split('::').last, detail: detail, kind: kind) } cache.set_constants(namespace, scope, result) result end |
#get_instance_methods(namespace, scope = '', visibility: [:public]) ⇒ Array<Suggestion>
214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 |
# File 'lib/solargraph/yard_map.rb', line 214 def get_instance_methods namespace, scope = '', visibility: [:public] cached = cache.get_instance_methods(namespace, scope, visibility) return cached unless cached.nil? meths = [] yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? ns = nil ns = find_first_resolved_namespace(yard, namespace, scope) unless ns.nil? ns.meths(scope: :instance, visibility: visibility).each { |m| n = m.to_s.split(/[\.#]/).last if n.to_s.match(/^[a-z]/i) and (namespace == 'Kernel' or !m.to_s.start_with?('Kernel#')) and !m.docstring.to_s.include?(':nodoc:') label = "#{n}" args = get_method_args(m) kind = (m.is_attribute? ? Suggestion::FIELD : Suggestion::METHOD) meths.push Suggestion.new(label, insert: "#{n.gsub(/=/, ' = ')}", kind: kind, documentation: m.docstring, code_object: m, detail: m.namespace, location: "#{m.file}:#{m.line}", arguments: args) end } if ns.kind_of?(YARD::CodeObjects::ClassObject) and namespace != 'Object' #if ns.superclass.kind_of?(YARD::CodeObjects::Proxy) unless ns.nil? meths += get_instance_methods(ns.superclass.to_s) end end ns.instance_mixins.each do |m| meths += get_instance_methods(m.to_s) unless m.to_s == 'Kernel' end end end } cache.set_instance_methods(namespace, scope, visibility, meths) meths end |
#get_methods(namespace, scope = '', visibility: [:public]) ⇒ Array<Suggestion>
163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 |
# File 'lib/solargraph/yard_map.rb', line 163 def get_methods namespace, scope = '', visibility: [:public] cached = cache.get_methods(namespace, scope, visibility) return cached unless cached.nil? meths = [] binds = [] yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? ns = nil ns = find_first_resolved_namespace(yard, namespace, scope) if namespace == '' and scope == '' # Check for a bind tag in the yardoc root. If it exists, treat # workspace code as a DSL that uses public instance methods in the # specified namespaces. b = yard.root.tag(:bind) binds.concat b.types unless b.nil? end unless ns.nil? ns.meths(scope: :class, visibility: visibility).each { |m| n = m.to_s.split(/[\.#]/).last.gsub(/=/, ' = ') label = "#{n}" args = get_method_args(m) kind = (m.is_attribute? ? Suggestion::FIELD : Suggestion::METHOD) meths.push Suggestion.new(label, insert: "#{n.gsub(/=/, ' = ')}", kind: kind, documentation: m.docstring, code_object: m, detail: "#{ns}", location: "#{m.file}:#{m.line}", arguments: args) } # Collect superclass methods if ns.kind_of?(YARD::CodeObjects::ClassObject) and !ns.superclass.nil? meths += get_methods ns.superclass.to_s, '', visibility: [:public, :protected] unless ['Object', 'BasicObject', ''].include?(ns.superclass.to_s) end if ns.kind_of?(YARD::CodeObjects::ClassObject) and namespace != 'Class' meths += get_instance_methods('Class') yard = load_yardoc(y) i = yard.at("#{ns}#initialize") unless i.nil? meths.delete_if{|m| m.label == 'new'} label = "#{i}" args = get_method_args(i) meths.push Suggestion.new('new', kind: Suggestion::METHOD, documentation: i.docstring, code_object: i, detail: "#{ns}", location: "#{i.file}:#{i.line}", arguments: args) end end end end } binds.each { |b| meths += get_instance_methods(b, scope, visibility: [:public]) } cache.set_methods(namespace, scope, visibility, meths) meths end |
#include_bundled_gems ⇒ Object
41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 |
# File 'lib/solargraph/yard_map.rb', line 41 def include_bundled_gems return if workspace.nil? lockfile = File.join(workspace, 'Gemfile.lock') return unless File.file?(lockfile) parser = Bundler::LockfileParser.new(Bundler.read_file(lockfile)) parser.specs.each do |s| STDERR.puts "Specs include #{s.name}" gy = YARD::Registry.yardoc_file_for_gem(s.name) if gy.nil? STDERR.puts "Bundled gem not found: #{s.name}" else yardocs.unshift gy unless yardocs.include?(gy) end end end |
#load_yardoc(y) ⇒ Object
62 63 64 65 66 67 68 69 70 71 72 73 74 |
# File 'lib/solargraph/yard_map.rb', line 62 def load_yardoc y begin if y.kind_of?(Array) YARD::Registry.load y, true else YARD::Registry.load! y end rescue Exception => e STDERR.puts "Error loading yardoc '#{y}' #{e.class} #{e.message}" yardocs.delete y nil end end |
#objects(path, space = '') ⇒ Object
264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 |
# File 'lib/solargraph/yard_map.rb', line 264 def objects path, space = '' result = [] yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? obj = find_first_resolved_namespace(yard, path, space) if obj.nil? and path.include?('#') parts = path.split('#') obj = yard.at(parts[0]) unless obj.nil? meths = obj.meths(scope: [:instance]).keep_if{|m| m.name.to_s == parts[1]} meths.each { |m| args = get_method_args(m) result.push Solargraph::Suggestion.new(m.name, kind: 'Method', detail: m.path, code_object: m, arguments: args) } end else unless obj.nil? args = [] args = get_method_args(obj) if obj.kind_of?(YARD::CodeObjects::MethodObject) kind = kind_of_object(obj) result.push Solargraph::Suggestion.new(obj.name, kind: kind, detail: obj.path, code_object: obj, arguments: args) end end end } result end |
#resolve(signature, scope) ⇒ Object
151 152 153 154 155 156 157 158 159 160 |
# File 'lib/solargraph/yard_map.rb', line 151 def resolve signature, scope yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? obj = yard.resolve(P(scope), signature) return obj unless obj.nil? end } nil end |
#search(query) ⇒ Object
77 78 79 80 81 82 83 84 85 86 87 88 |
# File 'lib/solargraph/yard_map.rb', line 77 def search query found = [] yardocs.each { |y| yard = load_yardoc(y) unless yard.nil? yard.paths.each { |p| found.push p if p.downcase.include?(query.downcase) } end } found.sort end |
#yardocs ⇒ Array<String>
58 59 60 |
# File 'lib/solargraph/yard_map.rb', line 58 def yardocs @yardocs ||= [] end |