Class: Solargraph::ApiMap
- Inherits:
-
Object
- Object
- Solargraph::ApiMap
- Extended by:
- Logging
- Includes:
- Logging
- Defined in:
- lib/solargraph/api_map.rb,
lib/solargraph/api_map/cache.rb,
lib/solargraph/api_map/index.rb,
lib/solargraph/api_map/store.rb,
lib/solargraph/api_map/constants.rb,
lib/solargraph/api_map/source_to_yard.rb
Overview
An aggregate provider for information about Workspaces, Sources, gems, and the Ruby core.
Defined Under Namespace
Modules: SourceToYard Classes: Cache, Constants, Index, Store
Constant Summary collapse
Constants included from Logging
Logging::DEFAULT_LOG_LEVEL, Logging::LOG_LEVELS
Instance Attribute Summary collapse
-
#loose_unions ⇒ Object
readonly
Returns the value of attribute loose_unions.
- #missing_docs ⇒ Array<String> readonly
- #unresolved_requires ⇒ Array<String> readonly
Class Method Summary collapse
-
.load(directory, loose_unions: true) ⇒ ApiMap
Create an ApiMap with a workspace in the specified directory.
-
.load_with_cache(directory, out = $stderr, loose_unions: true) ⇒ ApiMap
Create an ApiMap with a workspace in the specified directory and cache any missing gems.
- .reset_core(out: nil) ⇒ void
Instance Method Summary collapse
- #==(other) ⇒ Object
-
#bundled?(filename) ⇒ Boolean
True if the specified file was included in a bundle, i.e., it’s either included in a workspace or open in a library.
- #cache_all_for_doc_map!(out: $stderr, rebuild: false) ⇒ void
- #cache_gem(gemspec, rebuild: false, out: nil) ⇒ void
-
#catalog(bench) ⇒ self
Catalog a bench.
- #clip(cursor) ⇒ SourceMap::Clip
-
#clip_at(filename, position) ⇒ SourceMap::Clip
Get a clip by filename and position.
- #conventions_environ ⇒ Environ
- #core_pins ⇒ Enumerable<Pin::Base>
- #cursor_at(filename, position) ⇒ Source::Cursor
-
#dereference(pin) ⇒ String?
Get a fully qualified namespace from a reference pin.
- #doc_map ⇒ DocMap
-
#document(path) ⇒ Enumerable<Pin::Base>
deprecated
Deprecated.
This method is likely superfluous. Calling #get_path_pins directly should be sufficient.
-
#document_symbols(filename) ⇒ Array<Pin::Symbol>
Get an array of document symbols from a file.
- #eql?(other) ⇒ Boolean
- #get_block_pins ⇒ Enumerable<Solargraph::Pin::Block>
-
#get_class_variable_pins(namespace) ⇒ Enumerable<Solargraph::Pin::ClassVariable>
Get an array of class variable pins for a namespace.
-
#get_complex_type_methods(complex_type, context = '', internal = false) ⇒ Array<Solargraph::Pin::Base>
Get an array of method pins for a complex type.
-
#get_constants(namespace, *contexts) ⇒ Array<Solargraph::Pin::Constant, Solargraph::Pin::Namespace>
Get suggestions for constants in the specified namespace.
- #get_extends(fqns) ⇒ Array<Pin::Reference::Extend>
- #get_global_variable_pins ⇒ Enumerable<Solargraph::Pin::GlobalVariable>
- #get_includes(fqns) ⇒ Array<Pin::Reference::Include>
-
#get_instance_variable_pins(namespace, scope = :instance) ⇒ Array<Solargraph::Pin::InstanceVariable>
Get an array of instance variable pins defined in specified namespace and scope.
-
#get_method_stack(rooted_tag, name, scope: :instance, visibility: %i[private protected public],, preserve_generics: false) ⇒ Array<Solargraph::Pin::Method>
Get a stack of method pins for a method name in a potentially parameterized namespace.
-
#get_methods(rooted_tag, scope: :instance, visibility: [:public], deep: true) ⇒ Array<Solargraph::Pin::Method>
Get an array of methods available in a particular context.
- #get_namespace_pins(namespace, context) ⇒ Array<Pin::Namespace>
-
#get_path_pins(path) ⇒ Array<Pin::Base>
Get an array of pins that match the specified path.
-
#get_path_suggestions(path) ⇒ Array<Solargraph::Pin::Base>
deprecated
Deprecated.
Use #get_path_pins instead.
- #get_symbols ⇒ Enumerable<Solargraph::Pin::Base>
- #hash ⇒ Integer
- #index(pins) ⇒ self
-
#initialize(pins: [], loose_unions: true) ⇒ ApiMap
constructor
A new instance of ApiMap.
- #inner_get_methods_from_reference(fq_reference_tag, namespace_pin, type, scope, visibility, deep, skip, no_core) ⇒ Array<Pin::Base>
-
#inspect ⇒ Object
avoid enormous dump.
-
#keyword_pins ⇒ Enumerable<Solargraph::Pin::Keyword>
An array of pins based on Ruby keywords (‘if`, `end`, etc.).
- #locate_pins(location) ⇒ Array<Solargraph::Pin::Base>
-
#map(source, live: false) ⇒ self
Map a single source.
- #named_macro(name) ⇒ YARD::Tags::MacroDirective?
-
#namespace_exists?(name, context = '') ⇒ Boolean
True if the namespace exists.
- #pins ⇒ Array<Solargraph::Pin::Base>
-
#qualify(tag, *gates) ⇒ String?
Determine fully qualified tag for a given tag used inside the definition of another tag (“context”).
- #qualify_superclass(fq_sub_tag) ⇒ String?
-
#query_symbols(query) ⇒ Array<Pin::Base>
Get an array of all symbols in the workspace that match the query.
- #required ⇒ Set<String>
- #resolve(name, *gates) ⇒ String?
- #resolve_method_aliases(pins, visibility = %i[public private protected]) ⇒ Array<Pin::Base>
-
#search(query) ⇒ Array<String>
Get a list of documented paths that match the query.
-
#source_map(filename) ⇒ SourceMap
Get a source map by filename.
- #source_maps ⇒ Array<SourceMap>
-
#super_and_sub?(sup, sub) ⇒ Boolean
Check if a class is a superclass of another class.
- #to_s ⇒ Object
-
#type_include?(host_ns, module_ns) ⇒ Boolean
Check if the host class includes the specified module, ignoring type parameters used.
- #unalias(name) ⇒ ComplexType?
- #uncached_gemspecs ⇒ ::Array<Gem::Specification>
- #uncached_rbs_collection_gemspecs ⇒ ::Array<Gem::Specification>
- #uncached_yard_gemspecs ⇒ ::Array<Gem::Specification>
-
#var_at_location(candidates, name, closure, location) ⇒ Pin::BaseVariable?
Find a variable pin by name and where it is used.
- #workspace ⇒ Workspace?
Methods included from Logging
Constructor Details
Instance Attribute Details
#loose_unions ⇒ Object (readonly)
Returns the value of attribute loose_unions.
70 71 72 |
# File 'lib/solargraph/api_map.rb', line 70 def loose_unions @loose_unions end |
#missing_docs ⇒ Array<String> (readonly)
24 25 26 |
# File 'lib/solargraph/api_map.rb', line 24 def missing_docs @missing_docs end |
#unresolved_requires ⇒ Array<String> (readonly)
19 20 21 |
# File 'lib/solargraph/api_map.rb', line 19 def unresolved_requires @unresolved_requires end |
Class Method Details
.load(directory, loose_unions: true) ⇒ ApiMap
Create an ApiMap with a workspace in the specified directory.
198 199 200 201 202 203 204 205 206 |
# File 'lib/solargraph/api_map.rb', line 198 def self.load directory, loose_unions: true api_map = new(loose_unions: loose_unions) workspace = Solargraph::Workspace.new(directory) # api_map.catalog Bench.new(workspace: workspace) library = Library.new(workspace) library.map! api_map.catalog library.bench api_map end |
.load_with_cache(directory, out = $stderr, loose_unions: true) ⇒ ApiMap
Create an ApiMap with a workspace in the specified directory and cache any missing gems.
236 237 238 239 240 241 242 243 244 245 |
# File 'lib/solargraph/api_map.rb', line 236 def self.load_with_cache directory, out = $stderr, loose_unions: true api_map = load(directory, loose_unions: loose_unions) if api_map.uncached_gemspecs.empty? logger.info { "All gems cached for #{directory}" } return api_map end api_map.cache_all_for_doc_map!(out: out) load(directory, loose_unions: loose_unions) end |
Instance Method Details
#==(other) ⇒ Object
61 62 63 |
# File 'lib/solargraph/api_map.rb', line 61 def == other eql?(other) end |
#bundled?(filename) ⇒ Boolean
True if the specified file was included in a bundle, i.e., it’s either included in a workspace or open in a library.
665 666 667 |
# File 'lib/solargraph/api_map.rb', line 665 def bundled? filename source_map_hash.keys.include?(filename) end |
#cache_all_for_doc_map!(out: $stderr, rebuild: false) ⇒ void
This method returns an undefined value.
211 212 213 |
# File 'lib/solargraph/api_map.rb', line 211 def cache_all_for_doc_map! out: $stderr, rebuild: false doc_map.cache_all!(out, rebuild: rebuild) end |
#cache_gem(gemspec, rebuild: false, out: nil) ⇒ void
This method returns an undefined value.
219 220 221 |
# File 'lib/solargraph/api_map.rb', line 219 def cache_gem gemspec, rebuild: false, out: nil doc_map.cache(gemspec, rebuild: rebuild, out: out) end |
#catalog(bench) ⇒ self
Catalog a bench.
108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 |
# File 'lib/solargraph/api_map.rb', line 108 def catalog bench @source_map_hash = bench.source_map_hash iced_pins = bench.icebox.flat_map(&:pins) live_pins = bench.live_map&.all_pins || [] conventions_environ.clear source_map_hash.each_value do |map| conventions_environ.merge map.conventions_environ end unresolved_requires = (bench.external_requires + conventions_environ.requires + bench.workspace.config.required).to_a.compact.uniq recreate_docmap = @unresolved_requires != unresolved_requires || # @sg-ignore Unresolved call to rbs_collection_path on Solargraph::Workspace, nil workspace.rbs_collection_path != bench.workspace.rbs_collection_path || @doc_map.uncached_gemspecs.any? if recreate_docmap @doc_map = DocMap.new(unresolved_requires, bench.workspace, out: nil) # @todo Implement gem preferences @unresolved_requires = @doc_map.unresolved_requires end @cache.clear if store.update(@@core_map.pins, @doc_map.pins, conventions_environ.pins, iced_pins, live_pins) @missing_docs = [] # @todo Implement missing docs self end |
#clip(cursor) ⇒ SourceMap::Clip
632 633 634 635 636 |
# File 'lib/solargraph/api_map.rb', line 632 def clip cursor raise FileNotFoundError, "ApiMap did not catalog #{cursor.filename}" unless source_map_hash.key?(cursor.filename) SourceMap::Clip.new(self, cursor) end |
#clip_at(filename, position) ⇒ SourceMap::Clip
Get a clip by filename and position.
187 188 189 190 |
# File 'lib/solargraph/api_map.rb', line 187 def clip_at filename, position position = Position.normalize(position) clip(cursor_at(filename, position)) end |
#conventions_environ ⇒ Environ
169 170 171 |
# File 'lib/solargraph/api_map.rb', line 169 def conventions_environ @conventions_environ ||= Environ.new end |
#core_pins ⇒ Enumerable<Pin::Base>
152 153 154 |
# File 'lib/solargraph/api_map.rb', line 152 def core_pins @@core_map.pins end |
#cursor_at(filename, position) ⇒ Source::Cursor
176 177 178 179 180 |
# File 'lib/solargraph/api_map.rb', line 176 def cursor_at filename, position position = Position.normalize(position) raise FileNotFoundError, "File not found: #{filename}" unless source_map_hash.key?(filename) source_map_hash[filename].cursor_at(position) end |
#dereference(pin) ⇒ String?
Get a fully qualified namespace from a reference pin.
330 331 332 |
# File 'lib/solargraph/api_map.rb', line 330 def dereference pin store.constants.dereference(pin) end |
#doc_map ⇒ DocMap
132 133 134 |
# File 'lib/solargraph/api_map.rb', line 132 def doc_map @doc_map ||= DocMap.new([], Workspace.new('.')) end |
#document(path) ⇒ Enumerable<Pin::Base>
This method is likely superfluous. Calling #get_path_pins directly should be sufficient.
607 608 609 |
# File 'lib/solargraph/api_map.rb', line 607 def document path get_path_pins(path) end |
#document_symbols(filename) ⇒ Array<Pin::Symbol>
Get an array of document symbols from a file.
642 643 644 645 |
# File 'lib/solargraph/api_map.rb', line 642 def document_symbols filename return [] unless source_map_hash.key?(filename) # @todo Raise error? resolve_method_aliases source_map_hash[filename].document_symbols end |
#eql?(other) ⇒ Boolean
54 55 56 57 58 |
# File 'lib/solargraph/api_map.rb', line 54 def eql? other self.class == other.class && # @sg-ignore flow sensitive typing needs to handle self.class == other.class equality_fields == other.equality_fields end |
#get_block_pins ⇒ Enumerable<Solargraph::Pin::Block>
407 408 409 |
# File 'lib/solargraph/api_map.rb', line 407 def get_block_pins store.pins_by_class(Pin::Block) end |
#get_class_variable_pins(namespace) ⇒ Enumerable<Solargraph::Pin::ClassVariable>
Get an array of class variable pins for a namespace.
392 393 394 |
# File 'lib/solargraph/api_map.rb', line 392 def get_class_variable_pins namespace prefer_non_nil_variables(store.get_class_variables(namespace)) end |
#get_complex_type_methods(complex_type, context = '', internal = false) ⇒ Array<Solargraph::Pin::Base>
Get an array of method pins for a complex type.
The type’s namespace and the context should be fully qualified. If the context matches the namespace type or is a subclass of the type, protected methods are included in the results. If protected methods are included and internal is true, private methods are also included.
509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 |
# File 'lib/solargraph/api_map.rb', line 509 def get_complex_type_methods complex_type, context = '', internal = false # This method does not qualify the complex type's namespace because # it can cause conflicts between similar names, e.g., `Foo` vs. # `Other::Foo`. It still takes a context argument to determine whether # protected and private methods are visible. return [] if complex_type.undefined? || complex_type.void? result = Set.new complex_type.each do |type| if type.duck_type? result.add Pin::DuckMethod.new(name: type.to_s[1..], source: :api_map) result.merge get_methods('Object') else unless type.nil? || type.name == 'void' visibility = [:public] if type.namespace == context || super_and_sub?(type.namespace, context) visibility.push :protected visibility.push :private if internal end result.merge get_methods(type.tag, scope: type.scope, visibility: visibility) end end end result.to_a end |
#get_constants(namespace, *contexts) ⇒ Array<Solargraph::Pin::Constant, Solargraph::Pin::Namespace>
Get suggestions for constants in the specified namespace. The result may contain both constant and namespace pins.
280 281 282 283 284 285 286 287 288 289 |
# File 'lib/solargraph/api_map.rb', line 280 def get_constants namespace, *contexts namespace ||= '' gates = contexts.clone gates.push '' if contexts.empty? && namespace.empty? gates.push namespace unless namespace.empty? store.constants .collect(gates) .select { |pin| namespace.empty? || contexts.empty? || pin.namespace == namespace } .select { |pin| pin.visibility == :public || pin.namespace == namespace } end |
#get_extends(fqns) ⇒ Array<Pin::Reference::Extend>
336 337 338 |
# File 'lib/solargraph/api_map.rb', line 336 def get_extends fqns store.get_extends(fqns) end |
#get_global_variable_pins ⇒ Enumerable<Solargraph::Pin::GlobalVariable>
402 403 404 |
# File 'lib/solargraph/api_map.rb', line 402 def get_global_variable_pins store.pins_by_class(Pin::GlobalVariable) end |
#get_includes(fqns) ⇒ Array<Pin::Reference::Include>
342 343 344 |
# File 'lib/solargraph/api_map.rb', line 342 def get_includes fqns store.get_includes(fqns) end |
#get_instance_variable_pins(namespace, scope = :instance) ⇒ Array<Solargraph::Pin::InstanceVariable>
Get an array of instance variable pins defined in specified namespace and scope.
352 353 354 355 356 357 358 359 360 361 362 363 |
# File 'lib/solargraph/api_map.rb', line 352 def get_instance_variable_pins namespace, scope = :instance result = [] [namespace] result.concat store.get_instance_variables(namespace, scope) sc_fqns = namespace while (sc = store.get_superclass(sc_fqns)) # @sg-ignore flow sensitive typing needs to handle "if foo = bar" sc_fqns = store.constants.dereference(sc) result.concat store.get_instance_variables(sc_fqns, scope) end result end |
#get_method_stack(rooted_tag, name, scope: :instance, visibility: %i[private protected public],, preserve_generics: false) ⇒ Array<Solargraph::Pin::Method>
Get a stack of method pins for a method name in a potentially parameterized namespace. The order of the pins corresponds to the ancestry chain, with highest precedence first.
549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 |
# File 'lib/solargraph/api_map.rb', line 549 def get_method_stack rooted_tag, name, scope: :instance, visibility: %i[private protected public], preserve_generics: false rooted_type = ComplexType.parse(rooted_tag) fqns = rooted_type.namespace namespace_pin = store.get_path_pins(fqns).first methods = if namespace_pin.is_a?(Pin::Constant) type = namespace_pin.typify(self) type = namespace_pin.probe(self) unless type.defined? if type.defined? namespace_pin = store.get_path_pins(type.namespace).first get_methods(type.namespace, scope: scope, visibility: visibility).select { |p| p.name == name } else [] end else get_methods(rooted_tag, scope: scope, visibility: visibility).select { |p| p.name == name } end methods = erase_generics(namespace_pin, rooted_type, methods) unless preserve_generics methods end |
#get_methods(rooted_tag, scope: :instance, visibility: [:public], deep: true) ⇒ Array<Solargraph::Pin::Method>
Get an array of methods available in a particular context.
418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 |
# File 'lib/solargraph/api_map.rb', line 418 def get_methods rooted_tag, scope: :instance, visibility: [:public], deep: true if rooted_tag.start_with? 'Array(' # Array() are really tuples - use our fill, as the RBS repo # does not give us definitions for it rooted_tag = "Solargraph::Fills::Tuple(#{rooted_tag[6..-2]})" end rooted_type = ComplexType.try_parse(rooted_tag) fqns = rooted_type.namespace namespace_pin = store.get_path_pins(fqns).select { |p| p.is_a?(Pin::Namespace) }.first cached = cache.get_methods(rooted_tag, scope, visibility, deep) return cached.clone unless cached.nil? # @type [Array<Solargraph::Pin::Method>] result = [] skip = Set.new if rooted_tag == '' # @todo Implement domains conventions_environ.domains.each do |domain| type = ComplexType.try_parse(domain) next if type.undefined? result.concat inner_get_methods(type.name, type.scope, visibility, deep, skip) end result.concat inner_get_methods(rooted_tag, :class, visibility, deep, skip) result.concat inner_get_methods(rooted_tag, :instance, visibility, deep, skip) result.concat inner_get_methods('Kernel', :instance, visibility, deep, skip) else result.concat inner_get_methods(rooted_tag, scope, visibility, deep, skip) unless %w[Class Class<Class>].include?(rooted_tag) result.map! do |pin| next pin unless pin.path == 'Class#new' init_pin = get_method_stack(rooted_tag, 'initialize').first next pin unless init_pin type = ComplexType::SELF new_pin = Pin::Method.new( name: 'new', scope: :class, location: init_pin.location, return_type: type, comments: init_pin.comments, closure: init_pin.closure, source: init_pin.source, type_location: init_pin.type_location ) new_pin.parameters = init_pin.parameters.map do |init_param| param = init_param.clone param.closure = new_pin param.reset_generated! param end.freeze new_pin.signatures = init_pin.signatures.map do |init_sig| sig = init_sig.proxy(type) sig.parameters = init_sig.parameters.map do |param| param = param.clone param.closure = new_pin param.reset_generated! param end.freeze sig.closure = new_pin sig.reset_generated! sig end.freeze new_pin end end result.concat inner_get_methods('Kernel', :instance, [:public], deep, skip) if visibility.include?(:private) result.concat inner_get_methods('Module', scope, visibility, deep, skip) if scope == :module end result = resolve_method_aliases(result, visibility) if namespace_pin && rooted_tag != rooted_type.name result = result.map { |method_pin| method_pin.resolve_generics(namespace_pin, rooted_type) } end cache.set_methods(rooted_tag, scope, visibility, deep, result) result end |
#get_namespace_pins(namespace, context) ⇒ Array<Pin::Namespace>
294 295 296 |
# File 'lib/solargraph/api_map.rb', line 294 def get_namespace_pins namespace, context store.fqns_pins(qualify(namespace, context)) end |
#get_path_pins(path) ⇒ Array<Pin::Base>
Get an array of pins that match the specified path.
585 586 587 |
# File 'lib/solargraph/api_map.rb', line 585 def get_path_pins path get_path_suggestions(path) end |
#get_path_suggestions(path) ⇒ Array<Solargraph::Pin::Base>
Use #get_path_pins instead.
Get an array of all suggestions that match the specified path.
576 577 578 579 |
# File 'lib/solargraph/api_map.rb', line 576 def get_path_suggestions path return [] if path.nil? resolve_method_aliases store.get_path_pins(path) end |
#get_symbols ⇒ Enumerable<Solargraph::Pin::Base>
397 398 399 |
# File 'lib/solargraph/api_map.rb', line 397 def get_symbols store.get_symbols end |
#hash ⇒ Integer
66 67 68 |
# File 'lib/solargraph/api_map.rb', line 66 def hash equality_fields.hash end |
#index(pins) ⇒ self
83 84 85 86 87 88 89 90 91 |
# File 'lib/solargraph/api_map.rb', line 83 def index pins # @todo This implementation is incomplete. It should probably create a # Bench. @source_map_hash = {} conventions_environ.clear cache.clear store.update @@core_map.pins, pins self end |
#inner_get_methods_from_reference(fq_reference_tag, namespace_pin, type, scope, visibility, deep, skip, no_core) ⇒ Array<Pin::Base>
743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 760 761 762 763 764 765 |
# File 'lib/solargraph/api_map.rb', line 743 def inner_get_methods_from_reference fq_reference_tag, namespace_pin, type, scope, visibility, deep, skip, no_core logger.debug { "ApiMap#add_methods_from_reference(type=#{type}) starting" } # Ensure the types returned by the methods in the referenced # type are relative to the generic values passed in the # reference. e.g., Foo<String> might include Enumerable<String> # # @todo perform the same translation in the other areas # here after adding a spec and handling things correctly # in ApiMap::Store and RbsMap::Conversions for each resolved_reference_type = ComplexType.parse(fq_reference_tag).force_rooted.resolve_generics(namespace_pin, type) # @todo Can inner_get_methods be cached? Lots of lookups of base types going on. methods = inner_get_methods(resolved_reference_type.tag, scope, visibility, deep, skip, no_core) if namespace_pin && !resolved_reference_type.all_params.empty? reference_pin = store.get_path_pins(resolved_reference_type.name).select { |p| p.is_a?(Pin::Namespace) }.first # logger.debug { "ApiMap#add_methods_from_reference(type=#{type}) - resolving generics with #{reference_pin.generics}, #{resolved_reference_type.rooted_tags}" } methods = methods.map do |method_pin| method_pin.resolve_generics(reference_pin, resolved_reference_type) end end # logger.debug { "ApiMap#add_methods_from_reference(type=#{type}) - resolved_reference_type: #{resolved_reference_type} for type=#{type}: #{methods.map(&:name)}" } methods end |
#inspect ⇒ Object
avoid enormous dump
77 78 79 |
# File 'lib/solargraph/api_map.rb', line 77 def inspect to_s end |
#keyword_pins ⇒ Enumerable<Solargraph::Pin::Keyword>
An array of pins based on Ruby keywords (‘if`, `end`, etc.).
255 256 257 |
# File 'lib/solargraph/api_map.rb', line 255 def keyword_pins store.pins_by_class(Pin::Keyword) end |
#locate_pins(location) ⇒ Array<Solargraph::Pin::Base>
624 625 626 627 |
# File 'lib/solargraph/api_map.rb', line 624 def locate_pins location return [] if location.nil? || !source_map_hash.key?(location.filename) resolve_method_aliases source_map_hash[location.filename].locate_pins(location) end |
#map(source, live: false) ⇒ self
Map a single source.
98 99 100 101 102 |
# File 'lib/solargraph/api_map.rb', line 98 def map source, live: false map = Solargraph::SourceMap.map(source) catalog Bench.new(source_maps: [map], live_map: live ? map : nil) self end |
#named_macro(name) ⇒ YARD::Tags::MacroDirective?
158 159 160 161 |
# File 'lib/solargraph/api_map.rb', line 158 def named_macro name # @sg-ignore Need to add nil check here store.named_macros[name] end |
#namespace_exists?(name, context = '') ⇒ Boolean
True if the namespace exists.
270 271 272 |
# File 'lib/solargraph/api_map.rb', line 270 def namespace_exists? name, context = '' !qualify(name, context).nil? end |
#pins ⇒ Array<Solargraph::Pin::Base>
248 249 250 |
# File 'lib/solargraph/api_map.rb', line 248 def pins store.pins.clone.freeze end |
#qualify(tag, *gates) ⇒ String?
Determine fully qualified tag for a given tag used inside the definition of another tag (“context”). This method will start the search in the specified context until it finds a match for the tag.
Does not recurse into qualifying the type parameters, but returns any which were passed in unchanged.
313 314 315 |
# File 'lib/solargraph/api_map.rb', line 313 def qualify tag, *gates store.constants.qualify(tag, *gates) end |
#qualify_superclass(fq_sub_tag) ⇒ String?
769 770 771 |
# File 'lib/solargraph/api_map.rb', line 769 def qualify_superclass fq_sub_tag store.qualify_superclass fq_sub_tag end |
#query_symbols(query) ⇒ Array<Pin::Base>
Get an array of all symbols in the workspace that match the query.
615 616 617 618 619 620 |
# File 'lib/solargraph/api_map.rb', line 615 def query_symbols query Pin::Search.new( source_map_hash.values.flat_map(&:document_symbols), query ).results end |
#required ⇒ Set<String>
164 165 166 |
# File 'lib/solargraph/api_map.rb', line 164 def required @required ||= Set.new end |
#resolve(name, *gates) ⇒ String?
322 323 324 |
# File 'lib/solargraph/api_map.rb', line 322 def resolve name, *gates store.constants.resolve(name, *gates) end |
#resolve_method_aliases(pins, visibility = %i[public private protected]) ⇒ Array<Pin::Base>
713 714 715 716 717 718 719 720 721 722 723 724 725 |
# File 'lib/solargraph/api_map.rb', line 713 def resolve_method_aliases pins, visibility = %i[public private protected] with_resolved_aliases = pins.map do |pin| next pin unless pin.is_a?(Pin::MethodAlias) resolved = resolve_method_alias(pin) # @sg-ignore Need to add nil check here next nil if resolved.respond_to?(:visibility) && !visibility.include?(resolved.visibility) resolved end.compact logger.debug do "ApiMap#resolve_method_aliases(pins=#{pins.map(&:name)}, visibility=#{visibility}) => #{with_resolved_aliases.map(&:name)}" end GemPins.combine_method_pins_by_path(with_resolved_aliases) end |
#search(query) ⇒ Array<String>
Get a list of documented paths that match the query.
596 597 598 599 600 |
# File 'lib/solargraph/api_map.rb', line 596 def search query pins.map(&:path) .compact .select { |path| path.downcase.include?(query.downcase) } end |
#source_map(filename) ⇒ SourceMap
Get a source map by filename.
656 657 658 659 |
# File 'lib/solargraph/api_map.rb', line 656 def source_map filename raise FileNotFoundError, "Source map for `#{filename}` not found" unless source_map_hash.key?(filename) source_map_hash[filename] end |
#source_maps ⇒ Array<SourceMap>
648 649 650 |
# File 'lib/solargraph/api_map.rb', line 648 def source_maps source_map_hash.values end |
#super_and_sub?(sup, sub) ⇒ Boolean
Check if a class is a superclass of another class.
674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 690 691 692 693 694 695 696 697 |
# File 'lib/solargraph/api_map.rb', line 674 def super_and_sub? sup, sub sup = ComplexType.try_parse(sup) sub = ComplexType.try_parse(sub) # @todo If two literals are different values of the same type, it would # make more sense for super_and_sub? to return true, but there are a # few callers that currently expect this to be false. # @sg-ignore flow-sensitive typing should be able to handle redefinition return false if sup.literal? && sub.literal? && sup.to_s != sub.to_s # @sg-ignore flow sensitive typing should be able to handle redefinition sup = sup.simplify_literals.to_s # @sg-ignore flow sensitive typing should be able to handle redefinition sub = sub.simplify_literals.to_s return true if sup == sub sc_fqns = sub while (sc = store.get_superclass(sc_fqns)) # @sg-ignore flow sensitive typing needs to handle "if foo = bar" sc_new = store.constants.dereference(sc) # Cyclical inheritance is invalid return false if sc_new == sc_fqns sc_fqns = sc_new return true if sc_fqns == sup end false end |
#to_s ⇒ Object
72 73 74 |
# File 'lib/solargraph/api_map.rb', line 72 def to_s self.class.to_s end |
#type_include?(host_ns, module_ns) ⇒ Boolean
Check if the host class includes the specified module, ignoring type parameters used.
706 707 708 |
# File 'lib/solargraph/api_map.rb', line 706 def type_include? host_ns, module_ns store.get_includes(host_ns).map { |inc_tag| inc_tag.type.name }.include?(module_ns) end |
#unalias(name) ⇒ ComplexType?
261 262 263 |
# File 'lib/solargraph/api_map.rb', line 261 def unalias name store.unalias(name) end |
#uncached_gemspecs ⇒ ::Array<Gem::Specification>
137 138 139 |
# File 'lib/solargraph/api_map.rb', line 137 def uncached_gemspecs doc_map.uncached_gemspecs || [] end |
#uncached_rbs_collection_gemspecs ⇒ ::Array<Gem::Specification>
142 143 144 |
# File 'lib/solargraph/api_map.rb', line 142 def uncached_rbs_collection_gemspecs @doc_map.uncached_rbs_collection_gemspecs end |
#uncached_yard_gemspecs ⇒ ::Array<Gem::Specification>
147 148 149 |
# File 'lib/solargraph/api_map.rb', line 147 def uncached_yard_gemspecs @doc_map.uncached_yard_gemspecs end |
#var_at_location(candidates, name, closure, location) ⇒ Pin::BaseVariable?
Find a variable pin by name and where it is used.
Resolves our most specific view of this variable’s type by preferring pins created by flow-sensitive typing when we have them based on the Closure and Location.
377 378 379 380 381 382 383 384 385 386 |
# File 'lib/solargraph/api_map.rb', line 377 def var_at_location candidates, name, closure, location with_correct_name = candidates.select { |pin| pin.name == name } vars_at_location = with_correct_name.reject do |pin| # visible_at? excludes the starting position, but we want to # include it for this purpose !pin.visible_at?(closure, location) && !pin.starts_at?(location) end vars_at_location.inject(&:combine_with) end |
#workspace ⇒ Workspace?
728 729 730 |
# File 'lib/solargraph/api_map.rb', line 728 def workspace doc_map.workspace end |