Class: Solargraph::ApiMap
- Inherits:
-
Object
- Object
- Solargraph::ApiMap
- Includes:
- NodeMethods
- Defined in:
- lib/solargraph/api_map.rb,
lib/solargraph/api_map/config.rb
Defined Under Namespace
Classes: Config
Constant Summary collapse
- KEYWORDS =
[ '__ENCODING__', '__LINE__', '__FILE__', 'BEGIN', 'END', 'alias', 'and', 'begin', 'break', 'case', 'class', 'def', 'defined?', 'do', 'else', 'elsif', 'end', 'ensure', 'false', 'for', 'if', 'in', 'module', 'next', 'nil', 'not', 'or', 'redo', 'rescue', 'retry', 'return', 'self', 'super', 'then', 'true', 'undef', 'unless', 'until', 'when', 'while', 'yield' ]
- MAPPABLE_METHODS =
[ :include, :extend, :require, :autoload, :attr_reader, :attr_writer, :attr_accessor, :private, :public, :protected ]
Instance Attribute Summary collapse
-
#required ⇒ Object
readonly
Returns the value of attribute required.
-
#workspace ⇒ Object
readonly
Returns the value of attribute workspace.
Class Method Summary collapse
Instance Method Summary collapse
- #append_file(filename) ⇒ Object
- #append_node(node, comments, filename = nil) ⇒ Object
- #append_source(text, filename = nil) ⇒ Object
- #associate_comments(node, comments) ⇒ Object
- #clear ⇒ Object
- #find_fully_qualified_namespace(name, root = '', skip = []) ⇒ Object
- #find_instance_variable_assignment(var, node, scope) ⇒ Object
- #find_parent(node, *types) ⇒ Object
- #get_comment_for(node) ⇒ Object
- #get_filename_for(node) ⇒ Object
- #get_global_variables ⇒ Object
- #get_include_strings_from(*nodes) ⇒ Object
-
#get_instance_methods(namespace, root = '', visibility: [:public]) ⇒ Array<Solargraph::Suggestion>
Get an array of instance methods that are available in the specified namespace.
- #get_instance_variables(namespace, scope = :instance) ⇒ Object
- #get_method_args(node) ⇒ Object
-
#get_methods(namespace, root = '', visibility: [:public]) ⇒ Array<Solargraph::Suggestion>
Get an array of singleton methods that are available in the specified namespace.
- #get_namespace_nodes(fqns) ⇒ Object
- #get_namespace_type(namespace, root = '') ⇒ Object
- #get_root_for(node) ⇒ Object
- #get_superclass(namespace, root = '') ⇒ Object
- #infer_instance_variable(var, namespace, scope = :instance) ⇒ Object
-
#infer_signature_type(signature, namespace, scope: :instance) ⇒ String
Get a fully qualified namespace for the given signature.
-
#initialize(workspace = nil) ⇒ ApiMap
constructor
A new instance of ApiMap.
- #namespace_exists?(name, root = '') ⇒ Boolean
- #namespaces ⇒ Object
- #namespaces_in(name, root = '') ⇒ Object
- #process_maps ⇒ Object
-
#update_yardoc ⇒ Object
Update the YARD documentation for the current workspace.
- #yard_map ⇒ Solargraph::YardMap
- #yardoc_has_file?(file) ⇒ Boolean
Methods included from NodeMethods
#const_from, #infer, #pack_name, #resolve_node_signature, #stack_node_signature, #unpack_name, #yard_options
Constructor Details
#initialize(workspace = nil) ⇒ ApiMap
Returns a new instance of ApiMap.
28 29 30 31 32 33 34 35 36 37 38 39 |
# File 'lib/solargraph/api_map.rb', line 28 def initialize workspace = nil @workspace = workspace.gsub(/\\/, '/') unless workspace.nil? clear unless @workspace.nil? config = ApiMap::Config.new(@workspace) config.included.each { |f| unless config.excluded.include?(f) append_file f end } end end |
Instance Attribute Details
#required ⇒ Object (readonly)
Returns the value of attribute required.
25 26 27 |
# File 'lib/solargraph/api_map.rb', line 25 def required @required end |
#workspace ⇒ Object (readonly)
Returns the value of attribute workspace.
24 25 26 |
# File 'lib/solargraph/api_map.rb', line 24 def workspace @workspace end |
Class Method Details
.current ⇒ Object
417 418 419 420 421 422 423 |
# File 'lib/solargraph/api_map.rb', line 417 def self.current if @current.nil? @current = ApiMap.new @current.merge(Parser::CurrentRuby.parse(File.read("#{Solargraph::STUB_PATH}/ruby/2.3.0/core.rb"))) end @current end |
.get_keywords ⇒ Object
117 118 119 120 121 122 123 124 |
# File 'lib/solargraph/api_map.rb', line 117 def self.get_keywords result = [] keywords = KEYWORDS + ['attr_reader', 'attr_writer', 'attr_accessor', 'private', 'public', 'protected'] keywords.each { |k| result.push Suggestion.new(k, kind: Suggestion::KEYWORD, detail: 'Keyword') } result end |
Instance Method Details
#append_file(filename) ⇒ Object
57 58 59 |
# File 'lib/solargraph/api_map.rb', line 57 def append_file filename append_source File.read(filename), filename end |
#append_node(node, comments, filename = nil) ⇒ Object
74 75 76 77 78 79 80 81 82 83 |
# File 'lib/solargraph/api_map.rb', line 74 def append_node node, comments, filename = nil @file_comments[filename] = associate_comments(node, comments) mapified = reduce(node, @file_comments[filename]) root = AST::Node.new(:begin, [filename]) root = root.append mapified @file_nodes[filename] = root @required.uniq! process_maps root end |
#append_source(text, filename = nil) ⇒ Object
63 64 65 66 67 68 69 70 71 72 |
# File 'lib/solargraph/api_map.rb', line 63 def append_source text, filename = nil @file_source[filename] = text begin node, comments = Parser::CurrentRuby.parse_with_comments(text) append_node(node, comments, filename) rescue Parser::SyntaxError => e STDERR.puts "Error parsing '#{filename}': #{e.}" nil end end |
#associate_comments(node, comments) ⇒ Object
85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 |
# File 'lib/solargraph/api_map.rb', line 85 def associate_comments node, comments comment_hash = Parser::Source::Comment.associate_locations(node, comments) yard_hash = {} comment_hash.each_pair { |k, v| ctxt = '' num = nil started = false v.each { |l| # Trim the comment and minimum leading whitespace p = l.text.gsub(/^#/, '') if num.nil? and !p.strip.empty? num = p.index(/[^ ]/) started = true elsif started and !p.strip.empty? cur = p.index(/[^ ]/) num = cur if cur < num end if started ctxt += "#{p[num..-1]}\n" end } yard_hash[k] = YARD::Docstring.parser.parse(ctxt).to_docstring } yard_hash end |
#clear ⇒ Object
41 42 43 44 45 46 47 48 49 |
# File 'lib/solargraph/api_map.rb', line 41 def clear @file_source = {} @file_nodes = {} @file_comments = {} @parent_stack = {} @namespace_map = {} @namespace_tree = {} @required = [] end |
#find_fully_qualified_namespace(name, root = '', skip = []) ⇒ Object
158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 |
# File 'lib/solargraph/api_map.rb', line 158 def find_fully_qualified_namespace name, root = '', skip = [] return nil if skip.include?(root) skip.push root if name == '' if root == '' return '' else return find_fully_qualified_namespace(root, '', skip) end else if (root == '') return name unless @namespace_map[name].nil? get_include_strings_from(*@file_nodes.values).each { |i| reroot = "#{root == '' ? '' : root + '::'}#{i}" recname = find_fully_qualified_namespace name.to_s, reroot, skip return recname unless recname.nil? } else roots = root.to_s.split('::') while roots.length > 0 fqns = roots.join('::') + '::' + name return fqns unless @namespace_map[fqns].nil? roots.pop end return name unless @namespace_map[name].nil? get_include_strings_from(*@file_nodes.values).each { |i| recname = find_fully_qualified_namespace name, i, skip return recname unless recname.nil? } end end yard_map.find_fully_qualified_namespace(name, root) end |
#find_instance_variable_assignment(var, node, scope) ⇒ Object
266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 |
# File 'lib/solargraph/api_map.rb', line 266 def find_instance_variable_assignment(var, node, scope) node.children.each { |c| if c.kind_of?(AST::Node) is_inst = !find_parent(c, :def).nil? if c.type == :ivasgn and ( (scope == :instance and is_inst) or (scope != :instance and !is_inst) ) if c.children[0].to_s == var return c end else inner = find_instance_variable_assignment(var, c, scope) return inner unless inner.nil? end end } nil end |
#find_parent(node, *types) ⇒ Object
206 207 208 209 210 211 212 |
# File 'lib/solargraph/api_map.rb', line 206 def find_parent(node, *types) parents = @parent_stack[node] parents.each { |p| return p if types.include?(p.type) } nil end |
#get_comment_for(node) ⇒ Object
111 112 113 114 115 |
# File 'lib/solargraph/api_map.rb', line 111 def get_comment_for node filename = get_filename_for(node) return nil if @file_comments[filename].nil? @file_comments[filename][node.loc] end |
#get_filename_for(node) ⇒ Object
221 222 223 224 |
# File 'lib/solargraph/api_map.rb', line 221 def get_filename_for(node) root = get_root_for(node) root.nil? ? nil : root.children[0] end |
#get_global_variables ⇒ Object
283 284 285 286 |
# File 'lib/solargraph/api_map.rb', line 283 def get_global_variables # TODO: Get them [] end |
#get_include_strings_from(*nodes) ⇒ Object
425 426 427 428 429 430 431 432 433 434 435 |
# File 'lib/solargraph/api_map.rb', line 425 def get_include_strings_from *nodes arr = [] nodes.each { |node| next unless node.kind_of?(AST::Node) arr.push unpack_name(node.children[2]) if (node.type == :send and node.children[1] == :include) node.children.each { |n| arr += get_include_strings_from(n) if n.kind_of?(AST::Node) and n.type != :class and n.type != :module } } arr end |
#get_instance_methods(namespace, root = '', visibility: [:public]) ⇒ Array<Solargraph::Suggestion>
Get an array of instance methods that are available in the specified namespace.
386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 |
# File 'lib/solargraph/api_map.rb', line 386 def get_instance_methods(namespace, root = '', visibility: [:public]) meths = [] meths += inner_get_instance_methods(namespace, root, []) #unless has_yardoc? fqns = find_fully_qualified_namespace(namespace, root) yard_meths = yard_map.get_instance_methods(fqns, '', visibility: visibility) if yard_meths.any? meths.concat yard_meths else type = get_namespace_type(namespace, root) if type == :class meths += yard_map.get_instance_methods('Object') elsif type == :module meths += yard_map.get_instance_methods('Module') end end meths end |
#get_instance_variables(namespace, scope = :instance) ⇒ Object
197 198 199 200 201 202 203 204 |
# File 'lib/solargraph/api_map.rb', line 197 def get_instance_variables(namespace, scope = :instance) nodes = get_namespace_nodes(namespace) || @file_nodes.values arr = [] nodes.each { |n| arr += inner_get_instance_variables(n, scope) } arr end |
#get_method_args(node) ⇒ Object
358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 |
# File 'lib/solargraph/api_map.rb', line 358 def get_method_args node list = nil args = [] node.children.each { |c| if c.kind_of?(AST::Node) and c.type == :args list = c break end } return args if list.nil? list.children.each { |c| if c.type == :arg args.push c.children[0] elsif c.type == :optarg args.push "#{c.children[0]} = #{code_for(c.children[1])}" elsif c.type == :kwarg args.push "#{c.children[0]}:" elsif c.type == :kwoptarg args.push "#{c.children[0]}: #{code_for(c.children[1])}" end } args end |
#get_methods(namespace, root = '', visibility: [:public]) ⇒ Array<Solargraph::Suggestion>
Get an array of singleton methods that are available in the specified namespace.
341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 |
# File 'lib/solargraph/api_map.rb', line 341 def get_methods(namespace, root = '', visibility: [:public]) meths = [] meths += inner_get_methods(namespace, root, []) #unless has_yardoc? yard_meths = yard_map.get_methods(namespace, root, visibility: visibility) if yard_meths.any? meths.concat yard_meths else type = get_namespace_type(namespace, root) if type == :class meths += yard_map.get_instance_methods('Class') elsif type == :module meths += yard_map.get_methods('Module') end meths end end |
#get_namespace_nodes(fqns) ⇒ Object
192 193 194 195 |
# File 'lib/solargraph/api_map.rb', line 192 def get_namespace_nodes(fqns) return @file_nodes.values if fqns == '' or fqns.nil? @namespace_map[fqns] || [] end |
#get_namespace_type(namespace, root = '') ⇒ Object
327 328 329 330 331 332 333 334 335 |
# File 'lib/solargraph/api_map.rb', line 327 def get_namespace_type namespace, root = '' type = nil fqns = find_fully_qualified_namespace(namespace, root) nodes = get_namespace_nodes(fqns) unless nodes.nil? or nodes.empty? or !nodes[0].kind_of?(AST::Node) type = nodes[0].type if [:class, :module].include?(nodes[0].type) end type end |
#get_root_for(node) ⇒ Object
214 215 216 217 218 219 |
# File 'lib/solargraph/api_map.rb', line 214 def get_root_for(node) s = @parent_stack[node] return nil if s.nil? return node if s.empty? s.last end |
#get_superclass(namespace, root = '') ⇒ Object
404 405 406 407 408 409 410 411 412 413 414 415 |
# File 'lib/solargraph/api_map.rb', line 404 def get_superclass(namespace, root = '') fqns = find_fully_qualified_namespace(namespace, root) nodes = get_namespace_nodes(fqns) nodes.each { |n| if n.kind_of?(AST::Node) if n.type == :class and !n.children[1].nil? return unpack_name(n.children[1]) end end } return nil end |
#infer_instance_variable(var, namespace, scope = :instance) ⇒ Object
239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 |
# File 'lib/solargraph/api_map.rb', line 239 def infer_instance_variable(var, namespace, scope = :instance) result = nil vn = nil fqns = find_fully_qualified_namespace(namespace) unless fqns.nil? get_namespace_nodes(fqns).each { |node| vn = find_instance_variable_assignment(var, node, scope) break unless vn.nil? } end unless vn.nil? cmnt = get_comment_for(vn) unless cmnt.nil? tag = cmnt.tag(:type) result = tag.types[0] unless tag.nil? or tag.types.empty? end result = infer(vn.children[1]) if result.nil? if result.nil? signature = resolve_node_signature(vn.children[1]) sig_ns = find_fully_qualified_namespace(signature.split('.').first, fqns) sig_scope = (sig_ns.nil? ? :instance : :class) result = infer_signature_type(signature, namespace, scope: sig_scope) end end result end |
#infer_signature_type(signature, namespace, scope: :instance) ⇒ String
Get a fully qualified namespace for the given signature. The signature should be in the form of a method chain, e.g., method1.method2
294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 |
# File 'lib/solargraph/api_map.rb', line 294 def infer_signature_type signature, namespace, scope: :instance parts = signature.split('.') type = find_fully_qualified_namespace(namespace) type ||= '' top = true while parts.length > 0 and !type.nil? p = parts.shift if top and scope == :class next if p == 'new' first_class = find_fully_qualified_namespace(p, namespace) sub = nil sub = infer_signature_type(parts.join('.'), first_class, scope: :class) unless first_class.nil? return sub unless sub.to_s == '' end unless p == 'new' and scope != :instance if scope == :instance meths = get_instance_methods(type) meths += get_methods('') if top or type.to_s == '' else meths = get_methods(type) end meths.delete_if{ |m| m.insert != p } return nil if meths.empty? type = nil match = meths[0].return_type type = find_fully_qualified_namespace(match) unless match.nil? end scope = :instance top = false end type end |
#namespace_exists?(name, root = '') ⇒ Boolean
140 141 142 |
# File 'lib/solargraph/api_map.rb', line 140 def namespace_exists? name, root = '' !find_fully_qualified_namespace(name, root).nil? end |
#namespaces ⇒ Object
136 137 138 |
# File 'lib/solargraph/api_map.rb', line 136 def namespaces @namespace_map.keys end |
#namespaces_in(name, root = '') ⇒ Object
144 145 146 147 148 149 150 151 152 153 154 155 156 |
# File 'lib/solargraph/api_map.rb', line 144 def namespaces_in name, root = '' result = [] result += inner_namespaces_in(name, root, []) result += yard_map.get_constants name, root fqns = find_fully_qualified_namespace(name, root) unless fqns.nil? nodes = get_namespace_nodes(fqns) get_include_strings_from(*nodes).each { |i| result += yard_map.get_constants(i, root) } end result end |
#process_maps ⇒ Object
126 127 128 129 130 131 132 133 134 |
# File 'lib/solargraph/api_map.rb', line 126 def process_maps @parent_stack = {} @namespace_map = {} @namespace_tree = {} @file_nodes.values.each { |f| map_parents f map_namespaces f } end |
#update_yardoc ⇒ Object
Update the YARD documentation for the current workspace.
439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 |
# File 'lib/solargraph/api_map.rb', line 439 def update_yardoc if workspace.nil? STDERR.puts "No workspace specified for yardoc update." else Thread.new do Dir.chdir(workspace) do STDERR.puts "Updating the yardoc for #{workspace}..." cmd = "yardoc -e #{Solargraph::YARD_EXTENSION_FILE}" STDERR.puts "Update yardoc with #{cmd}" STDERR.puts `#{cmd}` unless $?.success? STDERR.puts "There was an error processing the workspace yardoc." end end end end end |
#yard_map ⇒ Solargraph::YardMap
52 53 54 |
# File 'lib/solargraph/api_map.rb', line 52 def yard_map @yard_map ||= YardMap.new(required: required, workspace: workspace) end |
#yardoc_has_file?(file) ⇒ Boolean
226 227 228 229 230 231 232 233 234 235 236 237 |
# File 'lib/solargraph/api_map.rb', line 226 def yardoc_has_file?(file) return false if workspace.nil? if @yardoc_files.nil? @yardoc_files = [] [:include].each { |glob| Dir[File.join workspace, glob].each { |f| @yardoc_files.push File.absolute_path(f) } } end @yardoc_files.include?(file) end |