Class: Solargraph::CodeMap

Inherits:
Object
  • Object
show all
Includes:
NodeMethods
Defined in:
lib/solargraph/code_map.rb

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from NodeMethods

#const_from, #infer_literal_node_type, #pack_name, #resolve_node_signature, #unpack_name

Constructor Details

#initialize(code: '', filename: nil, workspace: nil, api_map: nil, cursor: nil) ⇒ CodeMap

Returns a new instance of CodeMap.



29
30
31
32
33
34
35
36
37
38
39
40
# File 'lib/solargraph/code_map.rb', line 29

def initialize code: '', filename: nil, workspace: nil, api_map: nil, cursor: nil
  @workspace = workspace
  # HACK: Adjust incoming filename's path separator for yardoc file comparisons
  filename = filename.gsub(File::ALT_SEPARATOR, File::SEPARATOR) unless filename.nil? or File::ALT_SEPARATOR.nil?
  @filename = filename
  @api_map = api_map
  @source = self.api_map.virtualize filename, code, cursor
  @node = @source.node
  @code = @source.code
  @comments = @source.comments
  self.api_map.refresh
end

Instance Attribute Details

#codeString (readonly)

The source code being analyzed.

Returns:

  • (String)


14
15
16
# File 'lib/solargraph/code_map.rb', line 14

def code
  @code
end

#filenameString (readonly)

The filename for the source code.

Returns:

  • (String)


19
20
21
# File 'lib/solargraph/code_map.rb', line 19

def filename
  @filename
end

#nodeParser::AST::Node (readonly)

The root node of the parsed code.

Returns:

  • (Parser::AST::Node)


9
10
11
# File 'lib/solargraph/code_map.rb', line 9

def node
  @node
end

#workspaceString (readonly)

The root directory of the project. The ApiMap will search here for additional files to parse and analyze.

Returns:

  • (String)


25
26
27
# File 'lib/solargraph/code_map.rb', line 25

def workspace
  @workspace
end

Class Method Details

.get_offset(text, line, col) ⇒ Object



61
62
63
64
65
66
67
68
69
# File 'lib/solargraph/code_map.rb', line 61

def self.get_offset text, line, col
  offset = 0
  if line > 0
    text.lines[0..line - 1].each { |l|
      offset += l.length
    }
  end
  offset + col
end

Instance Method Details

#api_mapSolargraph::ApiMap

Get the associated ApiMap.

Returns:



45
46
47
# File 'lib/solargraph/code_map.rb', line 45

def api_map
  @api_map ||= ApiMap.new(workspace)
end

#comment_at?(index) ⇒ Boolean

Determine if the specified index is inside a comment.

Returns:

  • (Boolean)


102
103
104
105
106
107
# File 'lib/solargraph/code_map.rb', line 102

def comment_at?(index)
  @comments.each do |c|
    return true if index > c.location.expression.begin_pos and index <= c.location.expression.end_pos
  end
  false
end

#get_class_variables_at(index) ⇒ Array<Solargraph::Suggestion>

Returns:



177
178
179
180
# File 'lib/solargraph/code_map.rb', line 177

def get_class_variables_at(index)
  ns = namespace_at(index) || ''
  api_map.get_class_variables(ns)
end

#get_instance_variables_at(index) ⇒ Object



182
183
184
185
186
187
188
# File 'lib/solargraph/code_map.rb', line 182

def get_instance_variables_at(index)
  # @todo There are a lot of other cases that need to be handled here
  node = parent_node_from(index, :def, :defs, :class, :module, :sclass)
  ns = namespace_at(index) || ''
  scope = (node.type == :def ? :instance : :class)
  api_map.get_instance_variables(ns, scope)
end

#get_local_variables_and_methods_at(index) ⇒ Array<Solargraph::Suggestion>

Get an array of local variables and methods that can be accessed from the specified location in the code.

Parameters:

  • index (Integer)

Returns:



507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
# File 'lib/solargraph/code_map.rb', line 507

def get_local_variables_and_methods_at(index)
  result = []
  local = parent_node_from(index, :class, :module, :def, :defs) || @node
  #result += get_local_variables_from(local)
  result += get_local_variables_from(node_at(index))
  scope = namespace_at(index) || @node
  if local.type == :def
    result += api_map.get_instance_methods(scope, visibility: [:public, :private, :protected])
  else
    result += api_map.get_methods(scope, scope, visibility: [:public, :private, :protected])
  end
  if local.type == :def or local.type == :defs
    result += get_method_arguments_from local
  end
  result += get_yieldparams_at(index)
  # @todo This might not be necessary.
  #result += api_map.get_methods('Kernel')
  result
end

#get_offset(line, col) ⇒ Integer

Get the offset of the specified line and column. The offset (also called the “index”) is typically used to identify the cursor’s location in the code when generating suggestions. The line and column numbers should start at zero.

Parameters:

  • line (Integer)
  • col (Integer)

Returns:

  • (Integer)


57
58
59
# File 'lib/solargraph/code_map.rb', line 57

def get_offset line, col
  CodeMap.get_offset @code, line, col
end

#get_signature_at(index) ⇒ String

Get the signature at the specified index. A signature is a method call that can start with a constant, method, or variable and does not include any method arguments. Examples:

  • String.new -> String.new

  • @x.bar -> @x.bar

  • y.split(‘, ’).length -> y.split.length

Parameters:

  • index (Integer)

Returns:

  • (String)


475
476
477
# File 'lib/solargraph/code_map.rb', line 475

def get_signature_at index
  get_signature_data_at(index)[1]
end

#get_signature_index_at(index) ⇒ Object



479
480
481
# File 'lib/solargraph/code_map.rb', line 479

def get_signature_index_at index
  get_signature_data_at(index)[0]
end

#get_snippets_at(index) ⇒ Object



483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
# File 'lib/solargraph/code_map.rb', line 483

def get_snippets_at(index)
  result = []
  Snippets.definitions.each_pair { |name, detail|
    matched = false
    prefix = detail['prefix']
    while prefix.length > 0
      if @code[index-prefix.length, prefix.length] == prefix
        matched = true
        break
      end
      prefix = prefix[0..-2]
    end
    if matched
      result.push Suggestion.new(detail['prefix'], kind: Suggestion::SNIPPET, detail: name, insert: detail['body'].join("\r\n"))
    end
  }
  result
end

#get_type_comment(node) ⇒ Object



455
456
457
458
459
460
461
462
463
# File 'lib/solargraph/code_map.rb', line 455

def get_type_comment node
  obj = nil
  cmnt = @source.docstring_for(node)
  unless cmnt.nil?
    tag = cmnt.tag(:type)
    obj = tag.types[0] unless tag.nil? or tag.types.empty?
  end
  obj
end

#infer_signature_at(index) ⇒ String

Infer the type of the signature located at the specified index.

Examples:

# Given the following code:
nums = [1, 2, 3]
nums.join
# ...and given an index that points at the end of "nums.join",
# infer_signature_at will identify nums as an Array and the return
# type of Array#join as a String, so the signature's type will be
# String.

Returns:

  • (String)


309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
# File 'lib/solargraph/code_map.rb', line 309

def infer_signature_at index
  signature = get_signature_at(index)
  # Check for literals first
  return 'Integer' if signature.match(/^[0-9]+?\.?$/)
  literal = nil
  if (signature.empty? and @code[index - 1] == '.') or signature == '[].'
    literal = node_at(index - 2)
  elsif signature.start_with?('.')
    literal = node_at(index - 1)
  else
    beg_sig = get_signature_index_at(index)
    if @code[beg_sig] == '.'
      literal = node_at(beg_sig - 1)
    else
      literal = node_at(1 + beg_sig)
    end
  end
  type = infer_literal_node_type(literal)
  if type.nil?
    node = parent_node_from(index, :class, :module, :def, :defs) || @node
    result = infer_signature_from_node signature, node
    if result.nil? or result.empty?
      # The rest of this routine is dedicated to method and block parameters
      arg = nil
      if node.type == :def or node.type == :defs or node.type == :block
        # Check for method arguments
        parts = signature.split('.', 2)
        # @type [Solargraph::Suggestion]
        arg = get_method_arguments_from(node).keep_if{|s| s.to_s == parts[0] }.first
        unless arg.nil?
          if parts[1].nil?
            result = arg.return_type
          else
            result = api_map.infer_signature_type(parts[1], arg.return_type, scope: :instance)
          end
        end
      end
      if arg.nil?
        # Check for yieldparams
        parts = signature.split('.', 2)
        yp = get_yieldparams_at(index).keep_if{|s| s.to_s == parts[0]}.first
        unless yp.nil?
          if parts[1].nil? or parts[1].empty?
            result = yp.return_type
          else
            newsig = parts[1..-1].join('.')
            result = api_map.infer_signature_type(newsig, yp.return_type, scope: :instance)
          end
        end
      end
    end
  else
    if signature.empty? or signature == '[].'
      result = type
    else
      cursed = get_signature_index_at(index)
      if signature.start_with?('[].')
        rest = signature[3..-1]
      else
        if signature.start_with?('.')
          rest = signature[literal.loc.expression.end_pos+(cursed-literal.loc.expression.end_pos)..-1]
        else
          rest = signature
        end
      end
      return type if rest.nil?
      lit_code = @code[literal.loc.expression.begin_pos..literal.loc.expression.end_pos]
      rest = rest[lit_code.length..-1] if rest.start_with?(lit_code)
      rest = rest[1..-1] if rest.start_with?('.')
      rest = rest[0..-2] if rest.end_with?('.')
      if rest.empty?
        result = type
      else
        result = api_map.infer_signature_type(rest, type, scope: :instance)
      end
    end
  end
  result
end

#infer_signature_from_node(signature, node) ⇒ Object



398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
# File 'lib/solargraph/code_map.rb', line 398

def infer_signature_from_node signature, node
  inferred = nil
  parts = signature.split('.')
  ns_here = namespace_from(node)
  if parts[0] and parts[0].include?('::')
    sub = get_namespace_or_constant(parts[0], ns_here)
    unless sub.nil?
      return sub if signature.match(/^#{parts[0]}\.$/)
      parts[0] = sub
    end
  end
  unless signature.include?('.')
    fqns = api_map.find_fully_qualified_namespace(signature, ns_here)
    return "Class<#{fqns}>" unless fqns.nil?
  end
  start = parts[0]
  return nil if start.nil?
  remainder = parts[1..-1]
  if start.start_with?('@@')
    cv = api_map.get_class_variable_pins(ns_here).select{|s| s.name == start}.first
    return (cv.return_type || api_map.infer_assignment_node_type(cv.node, cv.namespace)) unless cv.nil?
  elsif start.start_with?('@')
    scope = (node.type == :def ? :instance : :class)
    iv = api_map.get_instance_variable_pins(ns_here, scope).select{|s| s.name == start}.first
    return (iv.return_type || api_map.infer_assignment_node_type(iv.node, iv.namespace)) unless iv.nil?
  end
  var = find_local_variable_node(start, node)
  if var.nil?
    scope = (node.type == :def ? :instance : :class)
    type = api_map.infer_signature_type(signature, ns_here, scope: scope)
    return type unless type.nil?
  else
    # Signature starts with a local variable
    type = get_type_comment(var)
    type = infer_literal_node_type(var.children[1]) if type.nil?
    if type.nil?
      vsig = resolve_node_signature(var.children[1])
      type = infer_signature_from_node vsig, node
    end
  end
  unless type.nil?
    if remainder[0] == 'new'
      remainder.shift
      if remainder.empty?
        inferred = type
      else
        inferred = api_map.infer_signature_type(remainder.join('.'), type, scope: :instance)
      end
    elsif remainder.empty?
      inferred = type
    else
      inferred = api_map.infer_signature_type(remainder.join('.'), type, scope: :instance)
    end
  end
  inferred
end

#local_variable_in_node?(name, node) ⇒ Boolean

Returns:

  • (Boolean)


389
390
391
392
393
394
395
396
# File 'lib/solargraph/code_map.rb', line 389

def local_variable_in_node?(name, node)
  return true unless find_local_variable_node(name, node).nil?
  if node.type == :def or node.type == :defs
    args = get_method_arguments_from(node).keep_if{|a| a.label == name}
    return true unless args.empty?
  end
  false
end

#namespace_at(index) ⇒ String

Get the namespace at the specified location. For example, given the code ‘class Foo; def bar; end; end`, index 14 (the center) is in the “Foo” namespace.

Returns:

  • (String)


130
131
132
133
134
135
136
137
138
139
140
141
142
# File 'lib/solargraph/code_map.rb', line 130

def namespace_at(index)
  tree = tree_at(index)
  return nil if tree.length == 0
  slice = tree
  parts = []
  slice.reverse.each { |n|
    if n.type == :class or n.type == :module
      c = const_from(n.children[0])
      parts.push c
    end
  }
  parts.join("::")
end

#namespace_from(node) ⇒ String

Get the namespace for the specified node. For example, given the code ‘class Foo; def bar; end; end`, the node for `def bar` is in the “Foo” namespace.

Returns:

  • (String)


149
150
151
152
153
154
155
# File 'lib/solargraph/code_map.rb', line 149

def namespace_from(node)
  if node.respond_to?(:loc)
    namespace_at(node.loc.expression.begin_pos)
  else
    ''
  end
end

#node_at(index) ⇒ AST::Node

Get the nearest node that contains the specified index.

Parameters:

  • index (Integer)

Returns:

  • (AST::Node)


87
88
89
# File 'lib/solargraph/code_map.rb', line 87

def node_at(index)
  tree_at(index).first
end

#parent_node_from(index, *types) ⇒ AST::Node

Find the nearest parent node from the specified index. If one or more types are provided, find the nearest node whose type is in the list.

Parameters:

  • index (Integer)
  • types (Array<Symbol>)

Returns:

  • (AST::Node)


115
116
117
118
119
120
121
122
123
# File 'lib/solargraph/code_map.rb', line 115

def parent_node_from(index, *types)
  arr = tree_at(index)
  arr.each { |a|
    if a.kind_of?(AST::Node) and (types.empty? or types.include?(a.type))
      return a
    end
  }
  @node
end

#resolve_object_at(index) ⇒ Object



259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
# File 'lib/solargraph/code_map.rb', line 259

def resolve_object_at index
  return [] if string_at?(index)
  signature = get_signature_at(index)
  cursor = index
  while @code[cursor] =~ /[a-z0-9_\?]/i
    signature += @code[cursor]
    cursor += 1
    break if cursor >= @code.length
  end
  return [] if signature.to_s == ''
  path = nil
  ns_here = namespace_at(index)
  node = parent_node_from(index, :class, :module, :def, :defs) || @node
  parts = signature.split('.')
  if parts.length > 1
    beginner = parts[0..-2].join('.')
    type = infer_signature_from_node(beginner, node)
    ender = parts.last
    path = "#{type}##{ender}"
  else
    if local_variable_in_node?(signature, node)
      path = infer_signature_from_node(signature, node)
    elsif signature.start_with?('@')
      path = api_map.infer_instance_variable(signature, ns_here, (node.type == :def ? :instance : :class))
    else
      path = signature
    end
    if path.nil?
      path = api_map.find_fully_qualified_namespace(signature, ns_here)
    end
  end
  return [] if path.nil?
  if path.start_with?('Class<')
    path.gsub!(/^Class<([a-z0-9_:]*)>#([a-z0-9_]*)$/i, '\\1.\\2')
  end
  api_map.get_path_suggestions(path)
end

#signatures_at(index) ⇒ Object



251
252
253
254
255
256
257
# File 'lib/solargraph/code_map.rb', line 251

def signatures_at index
  sig = signature_index_before(index)
  return [] if sig.nil?
  word = word_at(sig)
  sugg = suggest_at(sig - word.length)
  sugg.select{|s| s.label == word}
end

#string_at?(index) ⇒ Boolean

Determine if the specified index is inside a string.

Returns:

  • (Boolean)


94
95
96
97
# File 'lib/solargraph/code_map.rb', line 94

def string_at?(index)
  n = node_at(index)
  n.kind_of?(AST::Node) and n.type == :str
end

#suggest_at(index, filtered: true, with_snippets: false) ⇒ Array<Solargraph::Suggestion>

Get suggestions for code completion at the specified location in the source.

Returns:



194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
# File 'lib/solargraph/code_map.rb', line 194

def suggest_at index, filtered: true, with_snippets: false
  return [] if string_at?(index) or string_at?(index - 1) or comment_at?(index)
  signature = get_signature_at(index)
  unless signature.include?('.')
    if signature.start_with?(':')
      return api_map.get_symbols
    elsif signature.start_with?('@@')
      return get_class_variables_at(index)
    elsif signature.start_with?('@')
      return get_instance_variables_at(index)
    elsif signature.start_with?('$')
      return api_map.get_global_variables
    end
  end
  result = []
  type = nil
  if signature.include?('.')
    type = infer_signature_at(index)
    if type.nil? and signature.include?('.')
      last_period = @code[0..index].rindex('.')
      type = infer_signature_at(last_period)
    end
  end
  if type.nil?
    unless signature.include?('.')
      namespace = namespace_at(index)
      if signature.include?('::')
        parts = signature.split('::', -1)
        ns = parts[0..-2].join('::')
        result = api_map.namespaces_in(ns, namespace)
      else
        type = infer_literal_node_type(node_at(index - 2))
        if type.nil?
          current_namespace = namespace_at(index)
          parts = current_namespace.to_s.split('::')
          result += get_snippets_at(index) if with_snippets
          result += get_local_variables_and_methods_at(index)
          result += ApiMap.get_keywords
          while parts.length > 0
            ns = parts.join('::')
            result += api_map.namespaces_in(ns, namespace)
            parts.pop
          end
          result += api_map.namespaces_in('')
          result += api_map.get_instance_methods('Kernel')
        else
          result.concat api_map.get_instance_methods(type)
        end
      end
    end
  else
    result.concat api_map.get_instance_methods(type) unless (type == '' and signature.include?('.'))
  end
  result = reduce_starting_with(result, word_at(index)) if filtered
  result.uniq{|s| s.path}.sort{|a,b| a.label <=> b.label}
end

#tree_at(index) ⇒ Array<AST::Node>

Get an array of nodes containing the specified index, starting with the topmost node and ending with the nearest.

Parameters:

  • index (Integer)

Returns:

  • (Array<AST::Node>)


76
77
78
79
80
81
# File 'lib/solargraph/code_map.rb', line 76

def tree_at(index)
  arr = []
  arr.push @node
  inner_node_at(index, @node, arr)
  arr
end

#word_at(index) ⇒ String

Select the word that directly precedes the specified index. A word can only consist of letters, numbers, and underscores.

Parameters:

  • index (Integer)

Returns:

  • (String)


162
163
164
165
166
167
168
169
170
171
172
173
174
# File 'lib/solargraph/code_map.rb', line 162

def word_at index
  word = ''
  cursor = index - 1
  while cursor > -1
    char = @code[cursor, 1]
    break if char.nil? or char == ''
    word = char + word if char == '$'
    break unless char.match(/[a-z0-9_]/i)
    word = char + word
    cursor -= 1
  end
  word
end