Class: Solargraph::CodeMap

Inherits:
Object
  • Object
show all
Includes:
NodeMethods
Defined in:
lib/solargraph/code_map.rb

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from NodeMethods

#const_from, #infer_literal_node_type, #pack_name, #resolve_node_signature, #unpack_name

Constructor Details

#initialize(code: '', filename: nil, workspace: nil, api_map: nil, cursor: nil) ⇒ CodeMap

Returns a new instance of CodeMap.



35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
# File 'lib/solargraph/code_map.rb', line 35

def initialize code: '', filename: nil, workspace: nil, api_map: nil, cursor: nil
  @workspace = workspace
  # HACK: Adjust incoming filename's path separator for yardoc file comparisons
  filename = filename.gsub(File::ALT_SEPARATOR, File::SEPARATOR) unless filename.nil? or File::ALT_SEPARATOR.nil?
  @filename = filename
  @api_map = api_map
  @code = code.gsub(/\r/, '')
  tries = 0
  tmp = @code
  cursor = CodeMap.get_offset(@code, cursor[0], cursor[1]) if cursor.kind_of?(Array)
  fixed_cursor = false
  begin
    # HACK: The current file is parsed with a trailing underscore to fix
    # incomplete trees resulting from short scripts (e.g., a lone variable
    # assignment).
    node, @comments = Parser::CurrentRuby.parse_with_comments(tmp + "\n_")
    @node = self.api_map.append_node(node, @comments, filename)
    @parsed = tmp
    @code.freeze
    @parsed.freeze
  rescue Parser::SyntaxError => e
    if tries < 10
      tries += 1
      if tries == 10 and e.message.include?('token $end')
        tmp += "\nend"
      else
        if !fixed_cursor and !cursor.nil? and e.message.include?('token $end') and cursor >= 2
          fixed_cursor = true
          spot = cursor - 2
          if tmp[cursor - 1] == '.'
            repl = ';'
          else
            repl = '#'
          end
        else
          spot = e.diagnostic.location.begin_pos
          repl = '_'
          if tmp[spot] == '@' or tmp[spot] == ':'
            # Stub unfinished instance variables and symbols
            spot -= 1
          elsif tmp[spot - 1] == '.'
            # Stub unfinished method calls
            repl = '#' if spot == tmp.length or tmp[spot] == '\n'
            spot -= 2
          else
            # Stub the whole line
            spot = beginning_of_line_from(tmp, spot)
            repl = '#'
            if tmp[spot+1..-1].rstrip == 'end'
              repl= 'end;end'
            end
          end
        end
        tmp = tmp[0..spot] + repl + tmp[spot+repl.length+1..-1].to_s
      end
      retry
    end
    raise e
  end
end

Instance Attribute Details

#code ⇒ String (readonly)

The source code being analyzed.

Returns:

  • (String)


14
15
16
# File 'lib/solargraph/code_map.rb', line 14

def code
  @code
end

#filename ⇒ String (readonly)

The filename for the source code.

Returns:

  • (String)


25
26
27
# File 'lib/solargraph/code_map.rb', line 25

def filename
  @filename
end

#node ⇒ AST::Node

The root node of the parsed code.

Returns:

  • (AST::Node)


9
10
11
# File 'lib/solargraph/code_map.rb', line 9

def node
  @node
end

#parsed ⇒ String (readonly)

The source code after modification to fix syntax errors during parsing. This string will match #code if no modifications were made.

Returns:

  • (String)


20
21
22
# File 'lib/solargraph/code_map.rb', line 20

def parsed
  @parsed
end

#workspace ⇒ String (readonly)

The root directory of the project. The ApiMap will search here for additional files to parse and analyze.

Returns:

  • (String)


31
32
33
# File 'lib/solargraph/code_map.rb', line 31

def workspace
  @workspace
end

Class Method Details

.get_offset(text, line, col) ⇒ Object



115
116
117
118
119
120
121
122
123
# File 'lib/solargraph/code_map.rb', line 115

def self.get_offset text, line, col
  offset = 0
  if line > 0
    text.lines[0..line - 1].each { |l|
      offset += l.length
    }
  end
  offset + col      
end

Instance Method Details

#api_map ⇒ Solargraph::ApiMap

Get the ApiMap that was generated for this CodeMap.

Returns:



99
100
101
# File 'lib/solargraph/code_map.rb', line 99

def api_map
  @api_map ||= ApiMap.new(workspace)
end

#comment_at?(index) ⇒ Boolean

Determine if the specified index is inside a comment.

Returns:

  • (Boolean)


156
157
158
159
160
161
# File 'lib/solargraph/code_map.rb', line 156

def comment_at?(index)
  @comments.each do |c|
    return true if index > c.location.expression.begin_pos and index <= c.location.expression.end_pos
  end
  false
end

#get_class_variables_at(index) ⇒ Object



248
249
250
251
# File 'lib/solargraph/code_map.rb', line 248

def get_class_variables_at(index)
  ns = namespace_at(index) || ''
  api_map.get_class_variables(ns)
end

#get_instance_variables_at(index) ⇒ Object



253
254
255
256
257
# File 'lib/solargraph/code_map.rb', line 253

def get_instance_variables_at(index)
  node = parent_node_from(index, :def, :defs, :class, :module)
  ns = namespace_at(index) || ''
  api_map.get_instance_variables(ns, (node.type == :def ? :instance : :class))
end

#get_local_variables_and_methods_at(index) ⇒ Array<Solargraph::Suggestion>

Get an array of local variables and methods that can be accessed from the specified location in the code.

Parameters:

  • index (Integer)

Returns:



548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
# File 'lib/solargraph/code_map.rb', line 548

def get_local_variables_and_methods_at(index)
  result = []
  local = parent_node_from(index, :class, :module, :def, :defs) || @node
  result += get_local_variables_from(local)
  scope = namespace_at(index) || @node
  if local.type == :def
    result += api_map.get_instance_methods(scope, visibility: [:public, :private, :protected])
  else
    result += api_map.get_methods(scope, visibility: [:public, :private, :protected])
  end
  if local.type == :def or local.type == :defs
    result += get_method_arguments_from local
  end
  result += get_yieldparams_at(index)
  result += api_map.get_methods('Kernel')
  result
end

#get_offset(line, col) ⇒ Integer

Get the offset of the specified line and column. The offset (also called the "index") is typically used to identify the cursor's location in the code when generating suggestions. The line and column numbers should start at zero.

Parameters:

  • line (Integer)
  • col (Integer)

Returns:

  • (Integer)


111
112
113
# File 'lib/solargraph/code_map.rb', line 111

def get_offset line, col
  CodeMap.get_offset @code, line, col
end

#get_signature_at(index) ⇒ String

Get the signature at the specified index. A signature is a method call that can start with a constant, method, or variable and does not include any method arguments. Examples:

Code Signature

String.new String.new @x.bar @x.bar y.split(', ').length y.split.length

Parameters:

  • index (Integer)

Returns:

  • (String)


489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
# File 'lib/solargraph/code_map.rb', line 489

def get_signature_at index
  brackets = 0
  squares = 0
  parens = 0
  signature = ''
  index -=1
  while index >= 0
    break if brackets > 0 or parens > 0 or squares > 0
    char = @code[index, 1]
    if char == ')'
      parens -=1
    elsif char == ']'
      squares -=1
    elsif char == '}'
      brackets -= 1
    elsif char == '('
      parens += 1
    elsif char == '{'
      brackets += 1
    elsif char == '['
      squares += 1
    end
    if brackets == 0 and parens == 0 and squares == 0
      break if ['"', "'", ',', ' ', "\t", "\n"].include?(char)
      signature = char + signature if char.match(/[a-z0-9:\._@]/i)
      if char == '@'
        signature = "@#{signature}" if @code[index-1, 1] == '@'
        break
      end
    end
    index -= 1
  end
  signature
end

#get_snippets_at(index) ⇒ Object



524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
# File 'lib/solargraph/code_map.rb', line 524

def get_snippets_at(index)
  result = []
  Snippets.definitions.each_pair { |name, detail|
    matched = false
    prefix = detail['prefix']
    while prefix.length > 0
      if @code[index-prefix.length, prefix.length] == prefix
        matched = true
        break
      end
      prefix = prefix[0..-2]
    end
    if matched
      result.push Suggestion.new(detail['prefix'], kind: Suggestion::SNIPPET, detail: name, insert: detail['body'].join("\r\n"))
    end
  }
  result
end

#get_type_comment(node) ⇒ Object



467
468
469
470
471
472
473
474
475
# File 'lib/solargraph/code_map.rb', line 467

def get_type_comment node
  obj = nil
  cmnt = api_map.get_comment_for(node)
  unless cmnt.nil?
    tag = cmnt.tag(:type)
    obj = tag.types[0] unless tag.nil? or tag.types.empty?
  end
  obj
end

#infer_signature_at(index) ⇒ String

Infer the type of the signature located at the specified index.

Examples:

# Given the following code:
nums = [1, 2, 3]
nums.join
# ...and given an index that points at the end of "nums.join",
# infer_signature_at will identify nums as an Array and the return
# type of Array#join as a String, so the signature's type will be
# String.

Returns:

  • (String)


372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
# File 'lib/solargraph/code_map.rb', line 372

def infer_signature_at index
  signature = get_signature_at(index)
  node = parent_node_from(index, :class, :module, :def, :defs) || @node
  result = infer_signature_from_node signature, node
  if result.nil? or result.empty?
    arg = nil
    if node.type == :def or node.type == :defs or node.type == :block
      # Check for method arguments
      parts = signature.split('.', 2)
      # @type [Solargraph::Suggestion]
      arg = get_method_arguments_from(node).keep_if{|s| s.to_s == parts[0] }.first
      unless arg.nil?
        if parts[1].nil?
          result = arg.return_type
        else
          result = api_map.infer_signature_type(parts[1], parts[0], :instance)
        end
      end
    end
    if arg.nil?
      # Check for yieldparams
      parts = signature.split('.', 2)
      yp = get_yieldparams_at(index).keep_if{|s| s.to_s == parts[0]}.first
      unless yp.nil?
        if parts[1].nil? or parts[1].empty?
          result = yp.return_type
        else
          newsig = parts[1..-1].join('.')
          result = api_map.infer_signature_type(newsig, yp.return_type, scope: :instance)
        end
      end
    end
  end
  result
end

#infer_signature_from_node(signature, node) ⇒ Object



417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
# File 'lib/solargraph/code_map.rb', line 417

def infer_signature_from_node signature, node
  inferred = nil
  parts = signature.split('.')
  ns_here = namespace_from(node)
  start = parts[0]
  return nil if start.nil?
  remainder = parts[1..-1]
  if start.start_with?('@@')
    type = api_map.infer_class_variable(start, ns_here)
    return nil if type.nil?
    return type if remainder.empty?
    return api_map.infer_signature_type(remainder.join('.'), type, scope: :instance)
  elsif start.start_with?('@')
    scope = (node.type == :def ? :instance : :scope)
    type = api_map.infer_instance_variable(start, ns_here, scope)
    return nil if type.nil?
    return type if remainder.empty?
    return api_map.infer_signature_type(remainder.join('.'), type, scope: :instance)
  end
  var = find_local_variable_node(start, node)
  if var.nil?
    scope = (node.type == :def ? :instance : :class)
    type = api_map.infer_signature_type(signature, ns_here, scope: scope)
    return type unless type.nil?
  else
    # Signature starts with a local variable
    type = get_type_comment(var)
    type = infer_literal_node_type(var.children[1]) if type.nil?
    if type.nil?
      vsig = resolve_node_signature(var.children[1])
      type = infer_signature_from_node vsig, node
    end
  end
  unless type.nil?
    if remainder[0] == 'new'
      remainder.shift
      if remainder.empty?
        inferred = type
      else
        inferred = api_map.infer_signature_type(remainder.join('.'), type, scope: :instance)
      end
    elsif remainder.empty?
      inferred = type
    else
      inferred = api_map.infer_signature_type(remainder.join('.'), type, scope: :instance)
    end
  end
  inferred
end

#local_variable_in_node?(name, node) ⇒ Boolean

Returns:

  • (Boolean)


408
409
410
411
412
413
414
415
# File 'lib/solargraph/code_map.rb', line 408

def local_variable_in_node?(name, node)
  return true unless find_local_variable_node(name, node).nil?
  if node.type == :def or node.type == :defs
    args = get_method_arguments_from(node).keep_if{|a| a.label == name}
    return true unless args.empty?
  end
  false
end

#namespace_at(index) ⇒ String

Get the namespace at the specified location. For example, given the code class Foo; def bar; end; end, index 14 (the center) is in the "Foo" namespace.

Returns:

  • (String)


184
185
186
187
188
189
190
191
192
193
194
195
196
# File 'lib/solargraph/code_map.rb', line 184

def namespace_at(index)
  tree = tree_at(index)
  return nil if tree.length == 0
  slice = tree
  parts = []
  slice.reverse.each { |n|
    if n.type == :class or n.type == :module
      c = const_from(n.children[0])
      parts.push c
    end
  }
  parts.join("::")
end

#namespace_from(node) ⇒ String

Get the namespace for the specified node. For example, given the code class Foo; def bar; end; end, the node for def bar is in the "Foo" namespace.

Returns:

  • (String)


203
204
205
206
207
208
209
# File 'lib/solargraph/code_map.rb', line 203

def namespace_from(node)
  if node.respond_to?(:loc)
    namespace_at(node.loc.expression.begin_pos)
  else
    ''
  end
end

#node_at(index) ⇒ AST::Node

Get the nearest node that contains the specified index.

Parameters:

  • index (Integer)

Returns:

  • (AST::Node)


141
142
143
# File 'lib/solargraph/code_map.rb', line 141

def node_at(index)
  tree_at(index).first
end

#parent_node_from(index, *types) ⇒ AST::Node

Find the nearest parent node from the specified index. If one or more types are provided, find the nearest node whose type is in the list.

Parameters:

  • index (Integer)
  • types (Array<Symbol>)

Returns:

  • (AST::Node)


169
170
171
172
173
174
175
176
177
# File 'lib/solargraph/code_map.rb', line 169

def parent_node_from(index, *types)
  arr = tree_at(index)
  arr.each { |a|
    if a.kind_of?(AST::Node) and (types.empty? or types.include?(a.type))
      return a
    end
  }
  @node
end

#phrase_at(index) ⇒ String

Select the phrase that directly precedes the specified index. A phrase can consist of most types of characters other than whitespace, semi-colons, equal signs, parentheses, or brackets.

Parameters:

  • index (Integer)

Returns:

  • (String)


217
218
219
220
221
222
223
224
225
226
227
228
# File 'lib/solargraph/code_map.rb', line 217

def phrase_at index
  word = ''
  cursor = index - 1
  while cursor > -1
    char = @code[cursor, 1]
    break if char.nil? or char == ''
    break unless char.match(/[\s;=\(\)\[\]\{\}]/).nil?
    word = char + word
    cursor -= 1
  end
  word
end

#resolve_object_at(index) ⇒ Object



325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
# File 'lib/solargraph/code_map.rb', line 325

def resolve_object_at index
  return [] if string_at?(index)
  signature = get_signature_at(index)
  cursor = index
  while @code[cursor] =~ /[a-z0-9_\?]/i
    signature += @code[cursor]
    cursor += 1
    break if cursor >= @code.length
  end
  return [] if signature.to_s == ''
  path = nil
  ns_here = namespace_at(index)
  node = parent_node_from(index, :class, :module, :def, :defs) || @node
  parts = signature.split('.')
  if parts.length > 1
    beginner = parts[0..-2].join('.')
    type = infer_signature_from_node(beginner, node)
    ender = parts.last
    path = "#{type}##{ender}"
  else
    if local_variable_in_node?(signature, node)
      path = infer_signature_from_node(signature, node)
    elsif signature.start_with?('@')
      path = api_map.infer_instance_variable(signature, ns_here, (node.type == :def ? :instance : :class))
    else
      path = signature
    end
    if path.nil?
      path = api_map.find_fully_qualified_namespace(signature, ns_here)
    end
  end
  return [] if path.nil?
  return api_map.yard_map.objects(path, ns_here)
end

#signatures_at(index) ⇒ Object



318
319
320
321
322
323
# File 'lib/solargraph/code_map.rb', line 318

def signatures_at index
  sig = signature_index_before(index)
  return [] if sig.nil?
  word = word_at(sig)
  suggest_at(sig).reject{|s| s.label != word}
end

#string_at?(index) ⇒ Boolean

Determine if the specified index is inside a string.

Returns:

  • (Boolean)


148
149
150
151
# File 'lib/solargraph/code_map.rb', line 148

def string_at?(index)
  n = node_at(index)
  n.kind_of?(AST::Node) and n.type == :str
end

#suggest_at(index, filtered: false, with_snippets: false) ⇒ Array<Suggestions>

Get suggestions for code completion at the specified location in the source.

Returns:

  • (Array<Suggestions>) —

    The completion suggestions



263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
# File 'lib/solargraph/code_map.rb', line 263

def suggest_at index, filtered: false, with_snippets: false
  return [] if string_at?(index) or string_at?(index - 1) or comment_at?(index)
  result = []
  phrase = phrase_at(index)
  signature = get_signature_at(index)
  namespace = namespace_at(index)
  if signature.include?('.')
    # Check for literals first
    type = infer_literal_node_type(node_at(index - 2))
    if type.nil?
      nearest = @code[0, index].rindex('.')
      revised = signature[0..nearest-index-1]
      type = infer_signature_at(nearest) unless revised.empty?
      if !type.nil?
        result.concat api_map.get_instance_methods(type) unless type.nil?
      elsif !revised.include?('.')
        fqns = api_map.find_fully_qualified_namespace(revised, namespace)
        result.concat api_map.get_methods(fqns) unless fqns.nil?
      end
    else
      result.concat api_map.get_instance_methods(type)
    end
  elsif signature.start_with?('@@')
    result.concat get_class_variables_at(index)
  elsif signature.start_with?('@')
    result.concat get_instance_variables_at(index)
  elsif phrase.start_with?('$')
    result.concat api_map.get_global_variables
  elsif phrase.include?('::')
    parts = phrase.split('::', -1)
    ns = parts[0..-2].join('::')
    if parts.last.include?('.')
      ns = parts[0..-2].join('::') + '::' + parts.last[0..parts.last.index('.')-1]
      result = api_map.get_methods(ns)
    else
      result = api_map.namespaces_in(ns, namespace)
    end
  else
    current_namespace = namespace_at(index)
    parts = current_namespace.to_s.split('::')
    result += get_snippets_at(index) if with_snippets
    result += get_local_variables_and_methods_at(index)
    result += ApiMap.get_keywords
    while parts.length > 0
      ns = parts.join('::')
      result += api_map.namespaces_in(ns, namespace)
      parts.pop
    end
    result += api_map.namespaces_in('')
    result += api_map.get_instance_methods('Kernel')
  end
  result = reduce_starting_with(result, word_at(index)) if filtered
  result.uniq{|s| s.path}.sort{|a,b| a.label <=> b.label}
end

#tree_at(index) ⇒ Array<AST::Node>

Get an array of nodes containing the specified index, starting with the topmost node and ending with the nearest.

Parameters:

  • index (Integer)

Returns:

  • (Array<AST::Node>)


130
131
132
133
134
135
# File 'lib/solargraph/code_map.rb', line 130

def tree_at(index)
  arr = []
  arr.push @node
  inner_node_at(index, @node, arr)
  arr
end

#word_at(index) ⇒ String

Select the word that directly precedes the specified index. A word can only consist of letters, numbers, and underscores.

Parameters:

  • index (Integer)

Returns:

  • (String)


235
236
237
238
239
240
241
242
243
244
245
246
# File 'lib/solargraph/code_map.rb', line 235

def word_at index
  word = ''
  cursor = index - 1
  while cursor > -1
    char = @code[cursor, 1]
    break if char.nil? or char == ''
    break unless char.match(/[a-z0-9_]/i)
    word = char + word
    cursor -= 1
  end
  word
end