Class: Solargraph::ComplexType

Inherits:
Object
  • Object
show all
Defined in:
lib/solargraph/complex_type.rb,
lib/solargraph/complex_type/unique_type.rb,
lib/solargraph/complex_type/type_methods.rb

Overview

A container for type data based on YARD type tags.

Defined Under Namespace

Modules: TypeMethods Classes: UniqueType

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(types = [UniqueType::UNDEFINED]) ⇒ ComplexType

Returns a new instance of ComplexType.

Parameters:

  • types (Array<UniqueType>) (defaults to: [UniqueType::UNDEFINED])


14
15
16
# File 'lib/solargraph/complex_type.rb', line 14

def initialize types = [UniqueType::UNDEFINED]
  @items = types.uniq(&:to_s)
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(name, *args, &block) ⇒ Object



82
83
84
85
86
# File 'lib/solargraph/complex_type.rb', line 82

def method_missing name, *args, &block
  return if @items.first.nil?
  return @items.first.send(name, *args, &block) if respond_to_missing?(name)
  super
end

Class Method Details

.parse(*strings, partial: false) ⇒ ComplexType, ...

Note:

The ‘partial` parameter is used to indicate that the method is receiving a string that will be used inside another ComplexType. It returns arrays of ComplexTypes instead of a single cohesive one. Consumers should not need to use this parameter; it should only be used internally.

Parse type strings into a ComplexType.

Examples:

ComplexType.parse 'String', 'Foo', 'nil' #=> [String, Foo, nil]

Parameters:

  • *strings (Array<String>)

    The type definitions to parse

  • partial (Boolean) (defaults to: false)

    True if the string is part of a another type

Returns:



165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
# File 'lib/solargraph/complex_type.rb', line 165

def parse *strings, partial: false
  @cache ||= {}
  unless partial
    cached = @cache[strings]
    return cached unless cached.nil?
  end
  types = []
  key_types = nil
  strings.each do |type_string|
    point_stack = 0
    curly_stack = 0
    paren_stack = 0
    base = String.new
    subtype_string = String.new
    type_string&.each_char do |char|
      if char == '='
        #raise ComplexTypeError, "Invalid = in type #{type_string}" unless curly_stack > 0
      elsif char == '<'
        point_stack += 1
      elsif char == '>'
        if subtype_string.end_with?('=') && curly_stack > 0
          subtype_string += char
        elsif base.end_with?('=')
          raise ComplexTypeError, "Invalid hash thing" unless key_types.nil?
          # types.push ComplexType.new([UniqueType.new(base[0..-2].strip)])
          types.push UniqueType.new(base[0..-2].strip)
          key_types = types
          types = []
          base.clear
          subtype_string.clear
          next
        else
          raise ComplexTypeError, "Invalid close in type #{type_string}" if point_stack == 0
          point_stack -= 1
          subtype_string += char
        end
        next
      elsif char == '{'
        curly_stack += 1
      elsif char == '}'
        curly_stack -= 1
        subtype_string += char
        raise ComplexTypeError, "Invalid close in type #{type_string}" if curly_stack < 0
        next
      elsif char == '('
        paren_stack += 1
      elsif char == ')'
        paren_stack -= 1
        subtype_string += char if paren_stack == 0
        raise ComplexTypeError, "Invalid close in type #{type_string}" if paren_stack < 0
        next
      elsif char == ',' && point_stack == 0 && curly_stack == 0 && paren_stack == 0
        # types.push ComplexType.new([UniqueType.new(base.strip, subtype_string.strip)])
        types.push UniqueType.new(base.strip, subtype_string.strip)
        base.clear
        subtype_string.clear
        next
      end
      if point_stack == 0 && curly_stack == 0 && paren_stack == 0
        base.concat char
      else
        subtype_string.concat char
      end
    end
    raise ComplexTypeError, "Unclosed subtype in #{type_string}" if point_stack != 0 || curly_stack != 0 || paren_stack != 0
    # types.push ComplexType.new([UniqueType.new(base, subtype_string)])
    types.push UniqueType.new(base.strip, subtype_string.strip)
  end
  unless key_types.nil?
    raise ComplexTypeError, "Invalid use of key/value parameters" unless partial
    return key_types if types.empty?
    return [key_types, types]
  end
  result = partial ? types : ComplexType.new(types)
  @cache[strings] = result unless partial
  result
end

.try_parse(*strings) ⇒ ComplexType

Parameters:

  • strings (Array<String>)

Returns:



245
246
247
248
249
250
# File 'lib/solargraph/complex_type.rb', line 245

def try_parse *strings
  parse *strings
rescue ComplexTypeError => e
  Solargraph.logger.info "Error parsing complex type: #{e.message}"
  ComplexType::UNDEFINED
end

Instance Method Details

#[](index) ⇒ Object



63
64
65
# File 'lib/solargraph/complex_type.rb', line 63

def [](index)
  @items[index]
end

#all?(&block) ⇒ Boolean

Returns:

  • (Boolean)


96
97
98
# File 'lib/solargraph/complex_type.rb', line 96

def all? &block
  @items.all? &block
end

#all_paramsObject



130
131
132
# File 'lib/solargraph/complex_type.rb', line 130

def all_params
  @items.first.all_params || []
end

#any?(&block) ⇒ Boolean

Returns:

  • (Boolean)


100
101
102
# File 'lib/solargraph/complex_type.rb', line 100

def any? &block
  @items.compact.any? &block
end

#each {|| ... } ⇒ Array

Yield Parameters:

Returns:

  • (Array)


45
46
47
# File 'lib/solargraph/complex_type.rb', line 45

def each &block
  @items.each &block
end

#each_unique_type {|| ... } ⇒ Enumerator<UniqueType>

Yield Parameters:

Returns:



51
52
53
54
55
56
57
# File 'lib/solargraph/complex_type.rb', line 51

def each_unique_type &block
  return enum_for(__method__) unless block_given?

  @items.each do |item|
    item.each_unique_type &block
  end
end

#firstObject



29
30
31
# File 'lib/solargraph/complex_type.rb', line 29

def first
  @items.first
end

#lengthObject



59
60
61
# File 'lib/solargraph/complex_type.rb', line 59

def length
  @items.length
end

#map(&block) ⇒ Object



39
40
41
# File 'lib/solargraph/complex_type.rb', line 39

def map &block
  @items.map &block
end

#namespaceString

Returns:

  • (String)


72
73
74
75
# File 'lib/solargraph/complex_type.rb', line 72

def namespace
  # cache this attr for high frequency call
  @namespace ||= method_missing(:namespace).to_s
end

#namespacesArray<String>

Returns:

  • (Array<String>)


78
79
80
# File 'lib/solargraph/complex_type.rb', line 78

def namespaces
  @items.map(&:namespace)
end

#nullable?Boolean

Returns:

  • (Boolean)


126
127
128
# File 'lib/solargraph/complex_type.rb', line 126

def nullable?
  @items.any?(&:nil_type?)
end

#parameterized?Boolean

Returns:

  • (Boolean)


108
109
110
# File 'lib/solargraph/complex_type.rb', line 108

def parameterized?
  any?(&:parameterized?)
end

#qualify(api_map, context = '') ⇒ ComplexType

Parameters:

  • api_map (ApiMap)
  • context (String) (defaults to: '')

Returns:



21
22
23
24
25
26
27
# File 'lib/solargraph/complex_type.rb', line 21

def qualify api_map, context = ''
  types = @items.map do |t|
    next t if ['Boolean', 'nil', 'void', 'undefined'].include?(t.name)
    t.qualify api_map, context
  end
  ComplexType.new(types)
end

#resolve_parameters(definitions, context) ⇒ Object



112
113
114
115
# File 'lib/solargraph/complex_type.rb', line 112

def resolve_parameters definitions, context
  result = @items.map { |i| i.resolve_parameters(definitions, context) }
  ComplexType.parse(*result.map(&:tag))
end

#respond_to_missing?(name, include_private = false) ⇒ Boolean

Returns:

  • (Boolean)


88
89
90
# File 'lib/solargraph/complex_type.rb', line 88

def respond_to_missing?(name, include_private = false)
  TypeMethods.public_instance_methods.include?(name) || super
end

#select(&block) ⇒ Object



67
68
69
# File 'lib/solargraph/complex_type.rb', line 67

def select &block
  @items.select &block
end

#self_to(dst) ⇒ ComplexType

Parameters:

  • dst (String)

Returns:



119
120
121
122
123
124
# File 'lib/solargraph/complex_type.rb', line 119

def self_to dst
  return self unless selfy?
  red = reduce_class(dst)
  result = @items.map { |i| i.self_to red }
  ComplexType.parse(*result.map(&:to_s))
end

#selfy?Boolean

Returns:

  • (Boolean)


104
105
106
# File 'lib/solargraph/complex_type.rb', line 104

def selfy?
  @items.any?(&:selfy?)
end

#to_rbsObject



33
34
35
36
37
# File 'lib/solargraph/complex_type.rb', line 33

def to_rbs
  ((@items.length > 1 ? '(' : '') + @items.map do |item|
    "#{item.namespace}#{item.parameters? ? "[#{item.subtypes.map { |s| s.to_rbs }.join(', ')}]" : ''}"
  end.join(' | ') + (@items.length > 1 ? ')' : '')).gsub(/undefined/, 'untyped')
end

#to_sObject



92
93
94
# File 'lib/solargraph/complex_type.rb', line 92

def to_s
  map(&:tag).join(', ')
end