Class: Solargraph::ComplexType
- Inherits:
-
Object
- Object
- Solargraph::ComplexType
- Defined in:
- lib/solargraph/complex_type.rb,
lib/solargraph/complex_type/unique_type.rb,
lib/solargraph/complex_type/type_methods.rb
Overview
A container for type data based on YARD type tags.
Defined Under Namespace
Modules: TypeMethods Classes: UniqueType
Class Method Summary collapse
-
.parse(*strings, partial: false) ⇒ ComplexType, ...
Parse type strings into a ComplexType.
- .try_parse(*strings) ⇒ ComplexType
Instance Method Summary collapse
- #[](index) ⇒ Object
- #all?(&block) ⇒ Boolean
- #any?(&block) ⇒ Boolean
- #each {|| ... } ⇒ Array
- #each_unique_type {|| ... } ⇒ Enumerator<UniqueType>
- #first ⇒ Object
-
#initialize(types = [UniqueType::UNDEFINED]) ⇒ ComplexType
constructor
A new instance of ComplexType.
- #length ⇒ Object
- #map(&block) ⇒ Object
- #method_missing(name, *args, &block) ⇒ Object
- #namespace ⇒ Object
- #nullable? ⇒ Boolean
- #qualify(api_map, context = '') ⇒ ComplexType
- #respond_to_missing?(name, include_private = false) ⇒ Boolean
- #select(&block) ⇒ Object
- #self_to(dst) ⇒ ComplexType
- #selfy? ⇒ Boolean
- #to_s ⇒ Object
Constructor Details
#initialize(types = [UniqueType::UNDEFINED]) ⇒ ComplexType
Returns a new instance of ComplexType.
14 15 16 |
# File 'lib/solargraph/complex_type.rb', line 14 def initialize types = [UniqueType::UNDEFINED] @items = types.uniq(&:to_s) end |
Dynamic Method Handling
This class handles dynamic methods through the method_missing method
#method_missing(name, *args, &block) ⇒ Object
69 70 71 72 73 |
# File 'lib/solargraph/complex_type.rb', line 69 def method_missing name, *args, &block return if @items.first.nil? return @items.first.send(name, *args, &block) if respond_to_missing?(name) super end |
Class Method Details
.parse(*strings, partial: false) ⇒ ComplexType, ...
Note:
The `partial` parameter is used to indicate that the method is receiving a string that will be used inside another ComplexType. It returns arrays of ComplexTypes instead of a single cohesive one. Consumers should not need to use this parameter; it should only be used internally.
Parse type strings into a ComplexType.
139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 |
# File 'lib/solargraph/complex_type.rb', line 139 def parse *strings, partial: false @cache ||= {} unless partial cached = @cache[strings] return cached unless cached.nil? end types = [] key_types = nil strings.each do |type_string| point_stack = 0 curly_stack = 0 paren_stack = 0 base = String.new subtype_string = String.new type_string.each_char do |char| if char == '=' #raise ComplexTypeError, "Invalid = in type #{type_string}" unless curly_stack > 0 elsif char == '<' point_stack += 1 elsif char == '>' if subtype_string.end_with?('=') && curly_stack > 0 subtype_string += char elsif base.end_with?('=') raise ComplexTypeError, "Invalid hash thing" unless key_types.nil? # types.push ComplexType.new([UniqueType.new(base[0..-2].strip)]) types.push UniqueType.new(base[0..-2].strip) key_types = types types = [] base.clear subtype_string.clear next else raise ComplexTypeError, "Invalid close in type #{type_string}" if point_stack == 0 point_stack -= 1 subtype_string += char end next elsif char == '{' curly_stack += 1 elsif char == '}' curly_stack -= 1 subtype_string += char raise ComplexTypeError, "Invalid close in type #{type_string}" if curly_stack < 0 next elsif char == '(' paren_stack += 1 elsif char == ')' paren_stack -= 1 subtype_string += char if paren_stack == 0 raise ComplexTypeError, "Invalid close in type #{type_string}" if paren_stack < 0 next elsif char == ',' && point_stack == 0 && curly_stack == 0 && paren_stack == 0 # types.push ComplexType.new([UniqueType.new(base.strip, subtype_string.strip)]) types.push UniqueType.new(base.strip, subtype_string.strip) base.clear subtype_string.clear next end if point_stack == 0 && curly_stack == 0 && paren_stack == 0 base.concat char else subtype_string.concat char end end raise ComplexTypeError, "Unclosed subtype in #{type_string}" if point_stack != 0 || curly_stack != 0 || paren_stack != 0 # types.push ComplexType.new([UniqueType.new(base, subtype_string)]) types.push UniqueType.new(base.strip, subtype_string.strip) end unless key_types.nil? raise ComplexTypeError, "Invalid use of key/value parameters" unless partial return key_types if types.empty? return [key_types, types] end result = partial ? types : ComplexType.new(types) @cache[strings] = result unless partial result end |
.try_parse(*strings) ⇒ ComplexType
219 220 221 222 223 224 |
# File 'lib/solargraph/complex_type.rb', line 219 def try_parse *strings parse *strings rescue ComplexTypeError => e Solargraph.logger.info "Error parsing complex type: #{e.}" ComplexType::UNDEFINED end |
Instance Method Details
#[](index) ⇒ Object
57 58 59 |
# File 'lib/solargraph/complex_type.rb', line 57 def [](index) @items[index] end |
#all?(&block) ⇒ Boolean
83 84 85 |
# File 'lib/solargraph/complex_type.rb', line 83 def all? &block @items.all? &block end |
#any?(&block) ⇒ Boolean
87 88 89 |
# File 'lib/solargraph/complex_type.rb', line 87 def any? &block @items.any? &block end |
#each {|| ... } ⇒ Array
39 40 41 |
# File 'lib/solargraph/complex_type.rb', line 39 def each &block @items.each &block end |
#each_unique_type {|| ... } ⇒ Enumerator<UniqueType>
45 46 47 48 49 50 51 |
# File 'lib/solargraph/complex_type.rb', line 45 def each_unique_type &block return enum_for(__method__) unless block_given? @items.each do |item| item.each_unique_type &block end end |
#first ⇒ Object
29 30 31 |
# File 'lib/solargraph/complex_type.rb', line 29 def first @items.first end |
#length ⇒ Object
53 54 55 |
# File 'lib/solargraph/complex_type.rb', line 53 def length @items.length end |
#map(&block) ⇒ Object
33 34 35 |
# File 'lib/solargraph/complex_type.rb', line 33 def map &block @items.map &block end |
#namespace ⇒ Object
64 65 66 67 |
# File 'lib/solargraph/complex_type.rb', line 64 def namespace # cache this attr for high frequency call @namespace ||= method_missing(:namespace).to_s end |
#nullable? ⇒ Boolean
104 105 106 |
# File 'lib/solargraph/complex_type.rb', line 104 def nullable? @items.any?(&:nil_type?) end |
#qualify(api_map, context = '') ⇒ ComplexType
21 22 23 24 25 26 27 |
# File 'lib/solargraph/complex_type.rb', line 21 def qualify api_map, context = '' types = @items.map do |t| next t if ['Boolean', 'nil', 'void', 'undefined'].include?(t.name) t.qualify api_map, context end ComplexType.new(types) end |
#respond_to_missing?(name, include_private = false) ⇒ Boolean
75 76 77 |
# File 'lib/solargraph/complex_type.rb', line 75 def respond_to_missing?(name, include_private = false) TypeMethods.public_instance_methods.include?(name) || super end |
#select(&block) ⇒ Object
61 62 63 |
# File 'lib/solargraph/complex_type.rb', line 61 def select &block @items.select &block end |
#self_to(dst) ⇒ ComplexType
97 98 99 100 101 102 |
# File 'lib/solargraph/complex_type.rb', line 97 def self_to dst return self unless selfy? red = reduce_class(dst) result = @items.map { |i| i.self_to red } ComplexType.parse(*result.map(&:tag)) end |
#selfy? ⇒ Boolean
91 92 93 |
# File 'lib/solargraph/complex_type.rb', line 91 def selfy? @items.any?(&:selfy?) end |
#to_s ⇒ Object
79 80 81 |
# File 'lib/solargraph/complex_type.rb', line 79 def to_s map(&:tag).join(', ') end |