Module: Solargraph::Parser::ParserGem::NodeMethods
- Included in:
- NodeChainer, Solargraph::Parser::ParserGem::NodeProcessors::BlockNode, Solargraph::Parser::ParserGem::NodeProcessors::CasgnNode, Solargraph::Parser::ParserGem::NodeProcessors::DefsNode, Solargraph::Parser::ParserGem::NodeProcessors::IvasgnNode, Solargraph::Parser::ParserGem::NodeProcessors::LvasgnNode, Solargraph::Parser::ParserGem::NodeProcessors::NamespaceNode, Solargraph::Parser::ParserGem::NodeProcessors::ResbodyNode, Solargraph::Parser::ParserGem::NodeProcessors::SendNode
- Defined in:
- lib/solargraph/parser/parser_gem/node_methods.rb
Defined Under Namespace
Modules: DeepInference
Constant Summary collapse
- NIL_NODE =
::Parser::AST::Node.new(:nil)
Class Method Summary collapse
- .any_splatted_call?(nodes) ⇒ Boolean
- .call_nodes_from(node) ⇒ Array<Parser::AST::Node>
- .const_nodes_from(node) ⇒ Array<Parser::AST::Node>
- .convert_hash(node) ⇒ Hash{Parser::AST::Node => Chain}
- .drill_signature(node, signature) ⇒ String
- .find_recipient_node(cursor) ⇒ Parser::AST::Node?
- .get_node_end_position(node) ⇒ Position
- .get_node_start_position(node) ⇒ Position
- .infer_literal_node_type(node) ⇒ String?
- .pack_name(node) ⇒ Array<String>
- .repaired_find_recipient_node(cursor) ⇒ Parser::AST::Node?
-
.returns_from_method_body(node) ⇒ Array<Parser::AST::Node>
Find all the nodes within the provided node that potentially return a value.
- .splatted_call?(node) ⇒ Boolean
- .splatted_hash?(node) ⇒ Boolean
- .unpack_name(node) ⇒ String
-
.value_position_nodes_only(node) ⇒ Array<AST::Node>
Low-level value nodes in value position.
Class Method Details
.any_splatted_call?(nodes) ⇒ Boolean
162 163 164 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 162 def any_splatted_call?(nodes) nodes.any? { |n| splatted_call?(n) } end |
.call_nodes_from(node) ⇒ Array<Parser::AST::Node>
Temporarily here for testing. Move to Solargraph::Parser.
169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 169 def call_nodes_from node return [] unless node.is_a?(::Parser::AST::Node) result = [] if node.type == :block result.push node if Parser.is_ast_node?(node.children[0]) && node.children[0].children.length > 2 node.children[0].children[2..-1].each { |child| result.concat call_nodes_from(child) } end node.children[1..-1].each { |child| result.concat call_nodes_from(child) } elsif node.type == :send result.push node node.children[2..-1].each { |child| result.concat call_nodes_from(child) } elsif [:super, :zsuper].include?(node.type) result.push node node.children.each { |child| result.concat call_nodes_from(child) } elsif node.type == :masgn # @todo We're treating a mass assignment as a call node, but the # type checker still needs the logic to handle it. result.push node else node.children.each { |child| result.concat call_nodes_from(child) } end result end |
.const_nodes_from(node) ⇒ Array<Parser::AST::Node>
139 140 141 142 143 144 145 146 147 148 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 139 def const_nodes_from node return [] unless Parser.is_ast_node?(node) result = [] if node.type == :const result.push node else node.children.each { |child| result.concat const_nodes_from(child) } end result end |
.convert_hash(node) ⇒ Hash{Parser::AST::Node => Chain}
122 123 124 125 126 127 128 129 130 131 132 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 122 def convert_hash node return {} unless Parser.is_ast_node?(node) return convert_hash(node.children[0]) if node.type == :kwsplat return convert_hash(node.children[0]) if Parser.is_ast_node?(node.children[0]) && node.children[0].type == :kwsplat return {} unless node.type == :hash result = {} node.children.each do |pair| result[pair.children[0].children[0]] = Solargraph::Parser.chain(pair.children[1]) end result end |
.drill_signature(node, signature) ⇒ String
99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 99 def drill_signature node, signature return signature unless node.is_a?(AST::Node) if node.type == :const or node.type == :cbase unless node.children[0].nil? signature += drill_signature(node.children[0], signature) end signature += '::' unless signature.empty? signature += node.children[1].to_s elsif node.type == :lvar or node.type == :ivar or node.type == :cvar signature += '.' unless signature.empty? signature += node.children[0].to_s elsif node.type == :send unless node.children[0].nil? signature += drill_signature(node.children[0], signature) end signature += '.' unless signature.empty? signature += node.children[1].to_s end signature end |
.find_recipient_node(cursor) ⇒ Parser::AST::Node?
222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 222 def find_recipient_node cursor return repaired_find_recipient_node(cursor) if cursor.source.repaired? && cursor.source.code[cursor.offset - 1] == '(' source = cursor.source position = cursor.position offset = cursor.offset tree = if source.synchronized? match = source.code[0..offset-1].match(/,\s*\z/) if match source.tree_at(position.line, position.column - match[0].length) else source.tree_at(position.line, position.column) end else source.tree_at(position.line, position.column - 1) end prev = nil tree.each do |node| if node.type == :send args = node.children[2..-1] if !args.empty? return node if prev && args.include?(prev) else if source.synchronized? return node if source.code[0..offset-1] =~ /\(\s*\z/ && source.code[offset..-1] =~ /^\s*\)/ else return node if source.code[0..offset-1] =~ /\([^\(]*\z/ end end end prev = node end nil end |
.get_node_end_position(node) ⇒ Position
91 92 93 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 91 def get_node_end_position(node) Position.new(node.loc.last_line, node.loc.last_column) end |
.get_node_start_position(node) ⇒ Position
85 86 87 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 85 def get_node_start_position(node) Position.new(node.loc.line, node.loc.column) end |
.infer_literal_node_type(node) ⇒ String?
56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 56 def infer_literal_node_type node return nil unless node.is_a?(AST::Node) if node.type == :str || node.type == :dstr return '::String' elsif node.type == :array return '::Array' elsif node.type == :hash return '::Hash' elsif node.type == :int return '::Integer' elsif node.type == :float return '::Float' elsif node.type == :sym || node.type == :dsym return '::Symbol' elsif node.type == :regexp return '::Regexp' elsif node.type == :irange return '::Range' elsif node.type == :true || node.type == :false return '::Boolean' # @todo Support `nil` keyword in types # elsif node.type == :nil # return 'NilClass' end nil end |
.pack_name(node) ⇒ Array<String>
35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 35 def pack_name(node) # @type [Array<String>] parts = [] if node.is_a?(AST::Node) node.children.each { |n| if n.is_a?(AST::Node) if n.type == :cbase parts = [''] + pack_name(n) else parts += pack_name(n) end else parts.push n unless n.nil? end } end parts end |
.repaired_find_recipient_node(cursor) ⇒ Parser::AST::Node?
258 259 260 261 262 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 258 def repaired_find_recipient_node cursor cursor = cursor.source.cursor_at([cursor.position.line, cursor.position.column - 1]) node = cursor.source.tree_at(cursor.position.line, cursor.position.column).first return node if node && node.type == :send end |
.returns_from_method_body(node) ⇒ Array<Parser::AST::Node>
Find all the nodes within the provided node that potentially return a value.
The node parameter typically represents a method’s logic, e.g., the second child (after the :args node) of a :def node. A simple one-line method would typically return itself, while a node with conditions would return the resulting node from each conditional branch. Nodes that follow a :return node are assumed to be unreachable. Nil values are converted to nil node types.
206 207 208 209 210 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 206 def returns_from_method_body node # @todo is the || NIL_NODE necessary? # STDERR.puts("Evaluating expression: #{node.inspect}") DeepInference.from_method_body(node).map { |n| n || NIL_NODE } end |
.splatted_call?(node) ⇒ Boolean
156 157 158 159 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 156 def splatted_call? node return false unless Parser.is_ast_node?(node) Parser.is_ast_node?(node.children[0]) && node.children[0].type == :kwsplat && node.children[0].children[0].type != :hash end |
.splatted_hash?(node) ⇒ Boolean
151 152 153 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 151 def splatted_hash? node Parser.is_ast_node?(node.children[0]) && node.children[0].type == :kwsplat end |
.unpack_name(node) ⇒ String
29 30 31 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 29 def unpack_name(node) pack_name(node).join("::") end |
.value_position_nodes_only(node) ⇒ Array<AST::Node>
Returns low-level value nodes in value position. Does not include explicit return statements.
216 217 218 |
# File 'lib/solargraph/parser/parser_gem/node_methods.rb', line 216 def value_position_nodes_only(node) DeepInference.value_position_nodes_only(node).map { |n| n || NIL_NODE } end |