Class: Solargraph::Pin::Method

Inherits:
Closure show all
Includes:
ParserGem::NodeMethods
Defined in:
lib/solargraph/pin/method.rb

Overview

The base class for method and attribute pins.

Direct Known Subclasses

DelegatedMethod, DuckMethod, MethodAlias

Instance Attribute Summary collapse

Attributes inherited from Closure

#scope

Attributes inherited from Base

#code_object, #location, #name, #source

Attributes included from Common

#closure, #context, #location

Instance Method Summary collapse

Methods inherited from Closure

#binder, #context, #gates, #generics, #generics_as_rbs

Methods inherited from Base

#==, #comments, #deprecated?, #directives, #docstring, #erase_generics, #filename, #identity, #infer, #inspect, #macros, #maybe_directives?, #probed?, #proxied?, #proxy, #realize, #resolve_generics, #resolve_generics_from_context, #to_s, #variable?

Methods included from Documenting

normalize_indentation, strip_html_comments

Methods included from Conversions

#completion_item, #deprecated?, #link_documentation, #probed?, #proxied?, #reset_conversions, #resolve_completion_item, #text_documentation

Methods included from Common

#binder, #comments, #name, #namespace

Constructor Details

#initialize(visibility: :public, explicit: true, parameters: [], block: :undefined, node: nil, attribute: false, signatures: nil, anon_splat: false, return_type: nil, **splat) ⇒ Method

Returns a new instance of Method.

Parameters:

  • visibility (::Symbol) (defaults to: :public)

    :public, :protected, or :private

  • explicit (Boolean) (defaults to: true)
  • parameters (::Array<Pin::Parameter>) (defaults to: [])
  • block (Pin::Signature, nil, ::Symbol) (defaults to: :undefined)
  • node (Parser::AST::Node, RubyVM::AbstractSyntaxTree::Node, nil) (defaults to: nil)
  • attribute (Boolean) (defaults to: false)
  • signatures (::Array<Signature>, nil) (defaults to: nil)
  • anon_splat (Boolean) (defaults to: false)
  • return_type (ComplexType, nil) (defaults to: nil)


28
29
30
31
32
33
34
35
36
37
38
39
# File 'lib/solargraph/pin/method.rb', line 28

def initialize visibility: :public, explicit: true, parameters: [], block: :undefined, node: nil, attribute: false, signatures: nil, anon_splat: false, return_type: nil, **splat
  super(**splat)
  @visibility = visibility
  @explicit = explicit
  @parameters = parameters
  @block = block
  @node = node
  @attribute = attribute
  @signatures = signatures
  @anon_splat = anon_splat
  @return_type = return_type
end

Instance Attribute Details

#blockPin::Signature?

Returns:



75
76
77
78
# File 'lib/solargraph/pin/method.rb', line 75

def block
  return @block unless @block == :undefined
  @block = signatures.first.block
end

#documentationObject



202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
# File 'lib/solargraph/pin/method.rb', line 202

def documentation
  if @documentation.nil?
    @documentation ||= super || ''
    param_tags = docstring.tags(:param)
    unless param_tags.nil? or param_tags.empty?
      @documentation += "\n\n" unless @documentation.empty?
      @documentation += "Params:\n"
      lines = []
      param_tags.each do |p|
        l = "* #{p.name}"
        l += " [#{escape_brackets(p.types.join(', '))}]" unless p.types.nil? or p.types.empty?
        l += " #{p.text}"
        lines.push l
      end
      @documentation += lines.join("\n")
    end
    yieldparam_tags = docstring.tags(:yieldparam)
    unless yieldparam_tags.nil? or yieldparam_tags.empty?
      @documentation += "\n\n" unless @documentation.empty?
      @documentation += "Block Params:\n"
      lines = []
      yieldparam_tags.each do |p|
        l = "* #{p.name}"
        l += " [#{escape_brackets(p.types.join(', '))}]" unless p.types.nil? or p.types.empty?
        l += " #{p.text}"
        lines.push l
      end
      @documentation += lines.join("\n")
    end
    yieldreturn_tags = docstring.tags(:yieldreturn)
    unless yieldreturn_tags.empty?
      @documentation += "\n\n" unless @documentation.empty?
      @documentation += "Block Returns:\n"
      lines = []
      yieldreturn_tags.each do |r|
        l = "*"
        l += " [#{escape_brackets(r.types.join(', '))}]" unless r.types.nil? or r.types.empty?
        l += " #{r.text}"
        lines.push l
      end
      @documentation += lines.join("\n")
    end
    return_tags = docstring.tags(:return)
    unless return_tags.empty?
      @documentation += "\n\n" unless @documentation.empty?
      @documentation += "Returns:\n"
      lines = []
      return_tags.each do |r|
        l = "*"
        l += " [#{escape_brackets(r.types.join(', '))}]" unless r.types.nil? or r.types.empty?
        l += " #{r.text}"
        lines.push l
      end
      @documentation += lines.join("\n")
    end
    @documentation += "\n\n" unless @documentation.empty?
    @documentation += "Visibility: #{visibility}"
    concat_example_tags
  end
  @documentation.to_s
end

#nodeParser::AST::Node (readonly)

Returns:

  • (Parser::AST::Node)


17
18
19
# File 'lib/solargraph/pin/method.rb', line 17

def node
  @node
end

#parameters::Array<Pin::Parameter>

Returns:



11
12
13
# File 'lib/solargraph/pin/method.rb', line 11

def parameters
  @parameters
end

#signature_help::Array<Hash>

Returns:

  • (::Array<Hash>)


160
161
162
163
164
165
166
167
# File 'lib/solargraph/pin/method.rb', line 160

def signature_help
  @signature_help ||= signatures.map do |sig|
    {
      label: name + '(' + sig.parameters.map(&:full).join(', ') + ')',
      documentation: documentation
    }
  end
end

#signatures::Array<Signature>

Returns:



132
133
134
135
136
137
138
139
140
141
# File 'lib/solargraph/pin/method.rb', line 132

def signatures
  @signatures ||= begin
    top_type = generate_complex_type
    result = []
    result.push generate_signature(parameters, top_type) if top_type.defined?
    result.concat(overloads.map { |meth| generate_signature(meth.parameters, meth.return_type) }) unless overloads.empty?
    result.push generate_signature(parameters, @return_type || ComplexType::UNDEFINED) if result.empty?
    result
  end
end

#visibility::Symbol (readonly)

Returns :public, :private, or :protected.

Returns:

  • (::Symbol)

    :public, :private, or :protected



14
15
16
# File 'lib/solargraph/pin/method.rb', line 14

def visibility
  @visibility
end

Instance Method Details

#anon_splat?Boolean

Returns:

  • (Boolean)


312
313
314
# File 'lib/solargraph/pin/method.rb', line 312

def anon_splat?
  @anon_splat
end

#attribute?Boolean

Returns:

  • (Boolean)


268
269
270
# File 'lib/solargraph/pin/method.rb', line 268

def attribute?
  @attribute
end

#completion_item_kindObject



85
86
87
# File 'lib/solargraph/pin/method.rb', line 85

def completion_item_kind
  attribute? ? Solargraph::LanguageServer::CompletionItemKinds::PROPERTY : Solargraph::LanguageServer::CompletionItemKinds::METHOD
end

#descObject



169
170
171
172
173
174
175
176
# File 'lib/solargraph/pin/method.rb', line 169

def desc
  # ensure the signatures line up when logged
  if signatures.length > 1
    "\n#{to_rbs}\n"
  else
    to_rbs
  end
end

#detailString?

Returns:

  • (String, nil)


144
145
146
147
148
149
150
151
152
153
154
155
156
157
# File 'lib/solargraph/pin/method.rb', line 144

def detail
  # This property is not cached in an instance variable because it can
  # change when pins get proxied.
  detail = String.new
  detail += if signatures.length > 1
    "(*) "
  else
    "(#{signatures.first.parameters.map(&:full).join(', ')}) " unless signatures.first.parameters.empty?
  end.to_s
  detail += "=#{probed? ? '~' : (proxied? ? '^' : '>')} #{return_type.to_s}" unless return_type.undefined?
  detail.strip!
  return nil if detail.empty?
  detail
end

#explicit?Boolean

Returns:

  • (Boolean)


264
265
266
# File 'lib/solargraph/pin/method.rb', line 264

def explicit?
  @explicit
end

#generate_signature(parameters, return_type) ⇒ Signature

Parameters:

Returns:



100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
# File 'lib/solargraph/pin/method.rb', line 100

def generate_signature(parameters, return_type)
  block = nil
  yieldparam_tags = docstring.tags(:yieldparam)
  yieldreturn_tags = docstring.tags(:yieldreturn)
  generics = docstring.tags(:generic).map(&:name)
  needs_block_param_signature =
    parameters.last&.block? || !yieldreturn_tags.empty? || !yieldparam_tags.empty?
  if needs_block_param_signature
    yield_parameters = yieldparam_tags.map do |p|
      name = p.name
      decl = :arg
      if name
        decl = select_decl(name, false)
        name = clean_param(name)
      end
      Pin::Parameter.new(
        location: location,
        closure: self,
        comments: p.text,
        name: name,
        decl: decl,
        presence: location ? location.range : nil,
        return_type: ComplexType.try_parse(*p.types)
      )
    end
    yield_return_type = ComplexType.try_parse(*yieldreturn_tags.flat_map(&:types))
    block = Signature.new(generics, yield_parameters, yield_return_type)
  end
  Signature.new(generics, parameters, return_type, block)
end

#nearly?(other) ⇒ Boolean

Returns:

  • (Boolean)


272
273
274
275
276
277
# File 'lib/solargraph/pin/method.rb', line 272

def nearly? other
  return false unless super
  parameters == other.parameters and
    scope == other.scope and
    visibility == other.visibility
end

#overloads::Array<Pin::Method>

Returns:



290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
# File 'lib/solargraph/pin/method.rb', line 290

def overloads
  @overloads ||= docstring.tags(:overload).map do |tag|
    Pin::Signature.new(
      generics,
      tag.parameters.map do |src|
        name, decl = parse_overload_param(src.first)
        Pin::Parameter.new(
          location: location,
          closure: self,
          comments: tag.docstring.all.to_s,
          name: name,
          decl: decl,
          presence: location ? location.range : nil,
          return_type: param_type_from_name(tag, src.first)
        )
      end,
      ComplexType.try_parse(*tag.docstring.tags(:return).flat_map(&:types))
    )
  end
  @overloads
end

#parameter_names::Array<String>

Returns:

  • (::Array<String>)


81
82
83
# File 'lib/solargraph/pin/method.rb', line 81

def parameter_names
  @parameter_names ||= parameters.map(&:name)
end

#pathObject



190
191
192
# File 'lib/solargraph/pin/method.rb', line 190

def path
  @path ||= "#{namespace}#{(scope == :instance ? '#' : '.')}#{name}"
end

#probe(api_map) ⇒ Object



279
280
281
# File 'lib/solargraph/pin/method.rb', line 279

def probe api_map
  attribute? ? infer_from_iv(api_map) : infer_from_return_nodes(api_map)
end

#return_typeObject



93
94
95
# File 'lib/solargraph/pin/method.rb', line 93

def return_type
  @return_type ||= ComplexType.new(signatures.map(&:return_type).flat_map(&:items))
end

#symbol_kindObject



89
90
91
# File 'lib/solargraph/pin/method.rb', line 89

def symbol_kind
  attribute? ? Solargraph::LanguageServer::SymbolKinds::PROPERTY : LanguageServer::SymbolKinds::METHOD
end

#to_rbsObject



178
179
180
181
182
183
184
185
186
187
188
# File 'lib/solargraph/pin/method.rb', line 178

def to_rbs
  return nil if signatures.empty?

  rbs = "def #{name}: #{signatures.first.to_rbs}"
  signatures[1..].each do |sig|
    rbs += "\n"
    rbs += (' ' * (4 + name.length))
    rbs += "| #{name}: #{sig.to_rbs}"
  end
  rbs
end

#transform_types(&transform) ⇒ Object



41
42
43
44
45
46
47
48
49
50
51
52
53
54
# File 'lib/solargraph/pin/method.rb', line 41

def transform_types(&transform)
  # @todo 'super' alone should work here I think, but doesn't typecheck at level typed
  m = super(&transform)
  m.signatures = m.signatures.map do |sig|
    sig.transform_types(&transform)
  end
  m.parameters = m.parameters.map do |param|
    param.transform_types(&transform)
  end
  m.block = block&.transform_types(&transform)
  m.signature_help = nil
  m.documentation = nil
  m
end

#try_merge!(pin) ⇒ Object



283
284
285
286
287
# File 'lib/solargraph/pin/method.rb', line 283

def try_merge! pin
  return false unless super
  @node = pin.node
  true
end

#typify(api_map) ⇒ Object



194
195
196
197
198
199
200
# File 'lib/solargraph/pin/method.rb', line 194

def typify api_map
  decl = super
  return decl unless decl.undefined?
  type = see_reference(api_map) || typify_from_super(api_map)
  return type.qualify(api_map, namespace) unless type.nil?
  name.end_with?('?') ? ComplexType::BOOLEAN : ComplexType::UNDEFINED
end

#with_single_signature(signature) ⇒ Pin::Method

Parameters:

Returns:



58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
# File 'lib/solargraph/pin/method.rb', line 58

def with_single_signature(signature)
  m = proxy signature.return_type
  m.signature_help = nil
  m.documentation = nil
  # @todo populating the single parameters/return_type/block
  #   arguments here seems to be needed for some specs to pass,
  #   even though we have a signature with the same information.
  #   Is this a problem for RBS-populated methods, which don't
  #   populate these three?
  m.parameters = signature.parameters
  m.return_type = signature.return_type
  m.block = signature.block
  m.signatures = [signature]
  m
end