Class: Solargraph::SourceMap::Clip
- Inherits:
-
Object
- Object
- Solargraph::SourceMap::Clip
- Defined in:
- lib/solargraph/source_map/clip.rb
Overview
A static analysis tool for obtaining definitions, Completions, signatures, and type inferences from a Cursor.
Instance Method Summary collapse
- #complete ⇒ Completion
-
#define ⇒ Array<Pin::Base>
Relevant pins for infering the type of the Cursor’s position.
- #gates ⇒ ::Array<String>
- #in_block? ⇒ Boolean
- #infer ⇒ ComplexType
-
#initialize(api_map, cursor) ⇒ Clip
constructor
A new instance of Clip.
-
#locals ⇒ ::Array<Solargraph::Pin::Base>
Get an array of all the locals that are visible from the cursors’s position.
- #signify ⇒ Array<Pin::Base>
- #translate(phrase) ⇒ Array<Solargraph::Pin::Base>
- #types ⇒ Array<Pin::Base>
Constructor Details
Instance Method Details
#complete ⇒ Completion
32 33 34 35 36 37 38 39 40 41 |
# File 'lib/solargraph/source_map/clip.rb', line 32 def complete return package_completions([]) if !source_map.source.parsed? || cursor.string? return package_completions(api_map.get_symbols) if cursor.chain.literal? && cursor.chain.links.last.word == '<Symbol>' return Completion.new([], cursor.range) if cursor.chain.literal? if cursor.comment? tag_complete else code_complete end end |
#define ⇒ Array<Pin::Base>
Returns Relevant pins for infering the type of the Cursor’s position.
18 19 20 21 22 23 24 |
# File 'lib/solargraph/source_map/clip.rb', line 18 def define return [] if cursor.comment? || cursor.chain.literal? result = cursor.chain.define(api_map, block, locals) result.concat file_global_methods result.concat((source_map.pins + source_map.locals).select{ |p| p.name == cursor.word && p.location.range.contain?(cursor.position) }) if result.empty? result end |
#gates ⇒ ::Array<String>
72 73 74 |
# File 'lib/solargraph/source_map/clip.rb', line 72 def gates block.gates end |
#in_block? ⇒ Boolean
76 77 78 79 80 81 82 |
# File 'lib/solargraph/source_map/clip.rb', line 76 def in_block? return @in_block unless @in_block.nil? @in_block = begin tree = cursor.source.tree_at(cursor.position.line, cursor.position.column) Parser.is_ast_node?(tree[1]) && [:block, :ITER].include?(tree[1].type) end end |
#infer ⇒ ComplexType
51 52 53 54 55 56 57 58 59 60 |
# File 'lib/solargraph/source_map/clip.rb', line 51 def infer result = cursor.chain.infer(api_map, block, locals) if result.tag == 'Class' # HACK: Exception to return BasicObject from Class#new dfn = cursor.chain.define(api_map, block, locals).first return ComplexType.try_parse('::BasicObject') if dfn && dfn.path == 'Class#new' end return result unless result.tag == 'self' cursor.chain.base.infer(api_map, block, locals) end |
#locals ⇒ ::Array<Solargraph::Pin::Base>
Get an array of all the locals that are visible from the cursors’s position. Locals can be local variables, method parameters, or block parameters. The array starts with the nearest local pin.
67 68 69 |
# File 'lib/solargraph/source_map/clip.rb', line 67 def locals @locals ||= source_map.locals_at(location) end |
#signify ⇒ Array<Pin::Base>
44 45 46 47 48 |
# File 'lib/solargraph/source_map/clip.rb', line 44 def signify return [] unless cursor.argument? chain = Parser.chain(cursor.recipient_node, cursor.filename) chain.define(api_map, context_pin, locals).select { |pin| pin.is_a?(Pin::Method) } end |
#translate(phrase) ⇒ Array<Solargraph::Pin::Base>
86 87 88 89 |
# File 'lib/solargraph/source_map/clip.rb', line 86 def translate phrase chain = Parser.chain(Parser.parse(phrase)) chain.define(api_map, block, locals) end |
#types ⇒ Array<Pin::Base>
27 28 29 |
# File 'lib/solargraph/source_map/clip.rb', line 27 def types infer.namespaces.map { |namespace| api_map.get_path_pins(namespace) }.flatten end |